// Animace na GPU
// Vertex shader pro metodu SJBBE

int1	iBoneDebug = -1;
int1    ShowExVert = 0;
float4 lhtDir = {0.0f, 0.0f, -1.0f, 1.0f};    
float4 lightDiffuse = {0.6f, 0.6f, 0.6f, 1.0f};
float4 MaterialAmbient : MATERIALAMBIENT = {0.1f, 0.1f, 0.1f, 1.0f};
float4 MaterialDiffuse : MATERIALDIFFUSE = {0.8f, 0.8f, 0.8f, 1.0f};

static const int MAX_MATRICES = 50;
float4    WorldQuatArray[MAX_MATRICES];
float4	  WorldTrans[MAX_MATRICES];
float4    ExampleWeights[MAX_MATRICES];
float4x4    mViewProj : VIEWPROJECTION;

///////////////////////////////////////////////////////
struct VS_INPUT
{
    float3  Pos             : POSITION;
    float4  BlendWeights    : BLENDWEIGHT;
    float4  BlendIndices    : BLENDINDICES;
    float3  Normal          : NORMAL;
    float3  Tex0            : TEXCOORD0;
    float4  Pos1            : POSITION1;    
    float3  Pos2            : POSITION2;    
    float3  Pos3            : POSITION3;            
    float3  Pos4            : POSITION4;                
    float3  Norm1           : NORMAL1;    
    float3  Norm2           : NORMAL2;    
    float3  Norm3           : NORMAL3;            
    float3  Norm4           : NORMAL4;                
    
};

struct VS_OUTPUT
{
    float4  Pos     : POSITION;
    float4  Diffuse : COLOR;
    float2  Tex0    : TEXCOORD0;
};




float3x3 QuaternionToMatrix(float4 Quaternion)
{
	float3x3 Matrix = 0.0f;
	
	// Prevod quaternionu na matici
	float4 Quat2 = Quaternion + Quaternion;
	float4 Quat2a = Quat2.xyyy * Quaternion.xyzw;	// 2xx_2yy_2yz_2yw
	float4 Quat2b = Quat2.xxzz * Quaternion.ywzw;	// 2xy_2xw_2zz_2zw
	float4 Quat2plus2 = Quat2a.xxyy + Quat2b.zzzz;  // 2xx + 2zz, 2yy + 2zz
	
	Matrix[0][0] = 1 - Quat2plus2.z;			// 1 - (2yy + 2zz)
	Matrix[0][1] = Quat2b.x + Quat2b.w;			// 2xy + 2zw
	Matrix[0][2] = Quat2.x * Quaternion.z - Quat2a.w;	// 2xz - 2yw
	
	Matrix[1][0] = Quat2b.x - Quat2b.w;		// 2xy - 2zw
	Matrix[1][1] = 1 - Quat2plus2.x;		// 1 - (2xx + 2zz)
	Matrix[1][2] = Quat2a.z + Quat2b.y;		// 2yz + 2xw
	
	Matrix[2] = cross(Matrix[0],Matrix[1]);
	
	return Matrix;
	
}


float3 Diffuse(float3 Normal)
{
    float CosTheta;
    
    CosTheta = max(0.0f, dot(Normal, -lhtDir.xyz));
       
    return (CosTheta);
}


VS_OUTPUT VShade(VS_INPUT i, uniform int NumBones)
{
    VS_OUTPUT   o;
    float3      Pos = 0.0f;
    float3      Normal = 0.0f;    
    float       LastWeight = 0.0f;
    float4  	FinalQuat = 0.0f;
    float3x3	FinalMatrix = 0.0f;
    int			iBoneEx = (int) i.Pos1.w;
    
     
    int4 IndexVector = D3DCOLORtoUBYTE4(i.BlendIndices);

    // pretypovani vektoru na pole pro pouziti v cyklu
    float BlendWeightsArray[4] = (float[4])i.BlendWeights;
    int   IndexArray[4]        = (int[4])IndexVector;
    
    for (int iBone = 0; iBone < NumBones-1; iBone++)
    {
        LastWeight = LastWeight + BlendWeightsArray[iBone];
        FinalQuat += WorldQuatArray[IndexArray[iBone]]*BlendWeightsArray[iBone];

    }
    LastWeight = 1.0f - LastWeight; 

    FinalQuat += WorldQuatArray[IndexArray[NumBones-1]] * LastWeight;
    FinalQuat = normalize(FinalQuat);

    FinalMatrix = QuaternionToMatrix(FinalQuat);

    i.Pos.xyz += i.Pos1.xyz*ExampleWeights[iBoneEx].x;
    i.Pos.xyz += i.Pos2.xyz*ExampleWeights[iBoneEx].y;
    i.Pos.xyz += i.Pos3.xyz*ExampleWeights[iBoneEx].z;		
    i.Pos.xyz += i.Pos4.xyz*ExampleWeights[iBoneEx].w;

    i.Normal += i.Norm1*ExampleWeights[iBoneEx].x;
    i.Normal += i.Norm2*ExampleWeights[iBoneEx].y;
    i.Normal += i.Norm3*ExampleWeights[iBoneEx].z;		
    i.Normal += i.Norm4*ExampleWeights[iBoneEx].w;

    Pos = mul(i.Pos.xyz, FinalMatrix);
    Pos += WorldTrans[IndexArray[0]];

    Normal = mul(i.Normal, FinalMatrix);
    i.Normal = normalize(i.Normal);
    
    o.Pos = mul(float4(Pos.xyz, 1.0f), mViewProj);

    float3 ExColor = 0.0f;
    if ( ShowExVert == 1 && iBoneEx == iBoneDebug) ExColor.g = 1.0f;  

    o.Diffuse.xyz = MaterialAmbient.xyz + Diffuse(Normal) * MaterialDiffuse.xyz + ExColor.rgb;
    o.Diffuse.w = 1.0f;

    o.Tex0  = i.Tex0.xy;

    return o;
}

int CurNumBones = 3;
VertexShader vsArray[4] = { compile vs_2_0 VShade(1), 
                            compile vs_2_0 VShade(2),
                            compile vs_2_0 VShade(3),
                            compile vs_2_0 VShade(4)
                          };


technique t0
{
    pass p0
    {
        VertexShader = (vsArray[CurNumBones]);
    }
}

