
#include "dxstdafx.h"

#include "CApplication.h"
#include "CSkinningMethod.h"
#include "CMesh.h"
#include "CMeshMenu.h"


//#define DEBUG_VS   // Uncomment this line to debug vertex shaders 
//#define DEBUG_PS   // Uncomment this line to debug pixel shaders 



// Vsechny skinning metody, ke kazde SM je prirazeny nejaky typ modelu
CSkinningMethod*			CApplication::m_SkinningMethods[MAX_SKINNING_METHODS];
CSkinningMethod*			CApplication::m_ActualSM;
int							CApplication::m_iSMCount;

D3DCOLOR					CApplication::m_bgColor;

int							CApplication::m_NumVertices;
int							CApplication::m_NumFaces;


// Typy modelu
CVirtualMesh*				CApplication::m_Meshes[MAX_SKINNING_METHODS];
CVirtualMesh*				CApplication::m_ActualMesh;
int							CApplication::m_iMeshCount;

CD3DArcBall					CApplication::m_ArcBall;				// Arcball for camera control

bool						CApplication::m_bPaused;
bool						CApplication::m_bWireframe;
bool						CApplication::m_bShowHelp;

CDXUTDialog					CApplication::m_HUD;                  // dialog for standard controls
CDXUTDialog					CApplication::m_AnimationUI;
CDXUTDialog					CApplication::m_FreeMotionUI;

ID3DXFont*					CApplication::m_pFont;				// Font for drawing text
ID3DXSprite*				CApplication::m_pTextSprite;			// Sprite for batching draw text calls

D3DXMATRIXA16				CApplication::m_matView;              // View matrix
D3DXMATRIXA16				CApplication::m_matProj;              // Projection matrix


DWORD						CApplication::m_dwBehaviorFlags;      // Behavior flags of the 3D device
bool						CApplication::m_bUseSoftwareVP;		// Flag to indicate whether software vp is
																	// required due to lack of hardware
IDirect3DDevice9*			CApplication::m_pd3dDevice;



void CApplication::RegisterMethods(CSkinningMethod** methods, CVirtualMesh** meshes)
{

	m_bPaused = true;
	m_bWireframe = false;
	m_bShowHelp = true;
	m_pFont = NULL;
	m_pTextSprite = NULL;

	for (int i=0; i<MAX_SKINNING_METHODS; ++i)
		m_SkinningMethods[i] = NULL;

	for (int i=0; i<MAX_SKINNING_METHODS; ++i)
		m_Meshes[i] = NULL;


	for (int i=0;;i++)
	{
		if ( methods[i] == NULL ) 
		{
			m_iSMCount = i;
			break;
		}
		m_SkinningMethods[i] = methods[i];
	}

	for (int i=0;;i++)
	{
		if ( meshes[i] == NULL ) 
		{
			m_iMeshCount = i;
			break;
		}
		m_Meshes[i] = meshes[i];
	}

	m_ActualSM = m_SkinningMethods[0];
	m_ActualMesh = m_Meshes[m_SkinningMethods[0]->GetMeshID()];

	m_ActualMesh->SetSkinningMethod(m_ActualSM);
}

void	CApplication::ParseCommandLine(std::vector<std::wstring> &commands, LPWSTR strCmdLine)
{
	WCHAR* strToken;


	for(;;)
	{
		// Skip past any white space preceding the next token
		while (*strCmdLine && (*strCmdLine <= L' '))
			strCmdLine++;
		if( *strCmdLine == 0 )
			break;

		strToken = strCmdLine;
		while (*strCmdLine && (*strCmdLine != L' '))
			strCmdLine++;

		std::wstring strCommand(strToken,strCmdLine-strToken);
		commands.push_back(strCommand);

		if( *strCmdLine == 0 )
			break;
	}

	commands.push_back(L"");
	commands.push_back(L"");

}


HRESULT CApplication::Run()
{

	SetCallbackFunctions();

	// Pass control to the sample framework for handling the message pump and 
	// dispatching render calls. The sample framework will call your FrameMove 
	// and FrameRender callback when there is idle time between handling window messages.
	return DXUTMainLoop();

	// Perform any application-level cleanup here. Direct3D device resources are released within the
	// appropriate callback functions and therefore don't require any cleanup code here.

}

bool CApplication::IsDeviceAcceptable( D3DCAPS9* pCaps, D3DFORMAT AdapterFormat, 
								 D3DFORMAT BackBufferFormat, bool bWindowed )
{
	// Skip backbuffer formats that don't support alpha blending
	IDirect3D9* pD3D = DXUTGetD3DObject(); 
	if( FAILED( pD3D->CheckDeviceFormat( pCaps->AdapterOrdinal, pCaps->DeviceType,
		AdapterFormat, D3DUSAGE_QUERY_POSTPIXELSHADER_BLENDING, 
		D3DRTYPE_TEXTURE, BackBufferFormat ) ) )
		return false;

	return true;
}




void CApplication::SetCallbackFunctions()
{
	// Set the callback functions. These functions allow the sample framework to notify
	// the application about device changes, user input, and windows messages.  The 
	// callbacks are optional so you need only set callbacks for events you're interested 
	// in. However, if you don't handle the device reset/lost callbacks then the sample 
	// framework won't be able to reset your device since the application must first 
	// release all device resources before resetting.  Likewise, if you don't handle the 
	// device created/destroyed callbacks then the sample framework won't be able to 
	// recreate your device resources.

	/*	for(int i=0; i<D3DMAX; ++i)
	m_pEffects[i] = NULL;

	if (wcscmp((LPWSTR)cmdline,L"")) MESHFILENAME = cmdline;
	*/	DXUTSetCallbackDeviceCreated( OnCreateDevice );
	DXUTSetCallbackDeviceReset( OnResetDevice );
	DXUTSetCallbackDeviceLost( OnLostDevice );
	DXUTSetCallbackDeviceDestroyed( OnDestroyDevice );
	DXUTSetCallbackMsgProc( MsgProc );
	DXUTSetCallbackKeyboard( KeyboardProc );
	DXUTSetCallbackFrameRender( OnFrameRender );
	DXUTSetCallbackFrameMove( OnFrameMove );

	// Show the cursor and clip it when in full screen
	DXUTSetCursorSettings( true, true );

	InitUserInterface();

	// Initialize the sample framework and create the desired Win32 window and Direct3D 
	// device for the application. Calling each of these functions is optional, but they
	// allow you to set several options which control the behavior of the framework.
	DXUTInit( false, true, true ); // Parse the command line, handle the default hotkeys, and show msgboxes
	DXUTCreateWindow( L"Experimental skinning" );

	// Supports all types of vertex processing, including mixed.
	DXUTGetEnumeration()->SetPossibleVertexProcessingList( true, true, true, true );

	DXUTCreateDevice( D3DADAPTER_DEFAULT, true, 1024, 768, IsDeviceAcceptable, ModifyDeviceSettings );


}



//--------------------------------------------------------------------------------------
// This callback function is called immediately before a device is created to allow the 
// application to modify the device settings. The supplied pDeviceSettings parameter 
// contains the settings that the framework has selected for the new device, and the 
// application can make any desired changes directly to this structure.  Note however that 
// the sample framework will not correct invalid device settings so care must be taken 
// to return valid device settings, otherwise IDirect3D9::CreateDevice() will fail.  
//--------------------------------------------------------------------------------------
void CApplication::ModifyDeviceSettings( DXUTDeviceSettings* pDeviceSettings, const D3DCAPS9* pCaps )
{
	// If device doesn't support HW T&L or doesn't support 1.1 vertex shaders in HW 
	// then switch to SWVP.
	if( (pCaps->DevCaps & D3DDEVCAPS_HWTRANSFORMANDLIGHT) == 0 ||
		pCaps->VertexShaderVersion < D3DVS_VERSION(1,1) )
	{
		pDeviceSettings->BehaviorFlags = D3DCREATE_SOFTWARE_VERTEXPROCESSING;
	}
	else
	{
		pDeviceSettings->BehaviorFlags = D3DCREATE_HARDWARE_VERTEXPROCESSING;
	}

	// This application is designed to work on a pure device by not using 
	// IDirect3D9::Get*() methods, so create a pure device if supported and using HWVP.
	if ((pCaps->DevCaps & D3DDEVCAPS_PUREDEVICE) != 0 && 
		(pDeviceSettings->BehaviorFlags & D3DCREATE_HARDWARE_VERTEXPROCESSING) != 0 )
		pDeviceSettings->BehaviorFlags |= D3DCREATE_PUREDEVICE;

	// If the hardware cannot do vertex blending, use software vertex processing.
	if( pCaps->MaxVertexBlendMatrices < 2 )
		pDeviceSettings->BehaviorFlags = D3DCREATE_SOFTWARE_VERTEXPROCESSING;

	// If using hardware vertex processing, change to mixed vertex processing
	// so there is a fallback.
	//    if( pDeviceSettings->BehaviorFlags & D3DCREATE_HARDWARE_VERTEXPROCESSING )
	//      pDeviceSettings->BehaviorFlags = D3DCREATE_MIXED_VERTEXPROCESSING;    

	// Debugging vertex shaders requires either REF or software vertex processing 
	// and debugging pixel shaders requires REF.  
#ifdef DEBUG_VS
	if( pDeviceSettings->DeviceType != D3DDEVTYPE_REF )
	{
		pDeviceSettings->BehaviorFlags &= ~D3DCREATE_HARDWARE_VERTEXPROCESSING;
		pDeviceSettings->BehaviorFlags &= -D3DCREATE_MIXED_VERTEXPROCESSING;    
		pDeviceSettings->BehaviorFlags &= ~D3DCREATE_PUREDEVICE;
		pDeviceSettings->BehaviorFlags = D3DCREATE_SOFTWARE_VERTEXPROCESSING;
	}
#endif
#ifdef DEBUG_PS
	pDeviceSettings->DeviceType = D3DDEVTYPE_REF;
#endif
}




//--------------------------------------------------------------------------------------
// This callback function will be called immediately after the Direct3D device has been 
// created, which will happen during application initialization and windowed/full screen 
// toggles. This is the best location to create D3DPOOL_MANAGED resources since these 
// resources need to be reloaded whenever the device is destroyed. Resources created  
// here should be released in the OnDestroyDevice callback. 
//--------------------------------------------------------------------------------------
HRESULT CApplication::OnCreateDevice( IDirect3DDevice9* pd3dDevice, const D3DSURFACE_DESC* pBackBufferSurfaceDesc )
{
	HRESULT hr;

	m_pd3dDevice = pd3dDevice;
	// Initialize the font
	V_RETURN( D3DXCreateFont( pd3dDevice, 15, 0, FW_BOLD, 0, FALSE, DEFAULT_CHARSET, 
		OUT_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, 
		L"Arial", &m_pFont ) );

	// Define DEBUG_VS and/or DEBUG_PS to debug vertex and/or pixel shaders with the 
	// shader debugger. Debugging vertex shaders requires either REF or software vertex 
	// processing, and debugging pixel shaders requires REF.  The 
	// D3DXSHADER_FORCE_*_SOFTWARE_NOOPT flag improves the debug experience in the 
	// shader debugger.  It enables source level debugging, prevents instruction 
	// reordering, prevents dead code elimination, and forces the compiler to compile 
	// against the next higher available software target, which ensures that the 
	// unoptimized shaders do not exceed the shader model limitations.  Setting these 
	// flags will cause slower rendering since the shaders will be unoptimized and 
	// forced into software.  See the DirectX documentation for more information about 
	// using the shader debugger.
	DWORD dwShaderFlags = 0;
#ifdef DEBUG_VS
	dwShaderFlags |= D3DXSHADER_FORCE_VS_SOFTWARE_NOOPT | D3DXSHADER_SKIPOPTIMIZATION;
#endif
#ifdef DEBUG_PS
	dwShaderFlags |= D3DXSHADER_FORCE_PS_SOFTWARE_NOOPT | D3DXSHADER_SKIPOPTIMIZATION;
#endif



	// Obtain the behavior flags
	D3DDEVICE_CREATION_PARAMETERS cp;
	pd3dDevice->GetCreationParameters( &cp );
	m_dwBehaviorFlags = cp.BehaviorFlags;

	// Load the mesh
	for (int i=0; i<m_iMeshCount; ++i)
		m_Meshes[i]->OnCreateDevice(pd3dDevice, m_dwBehaviorFlags);

	for (int i=0; i<m_iSMCount; ++i)
		m_SkinningMethods[i]->OnCreateDevice(pd3dDevice, dwShaderFlags);



	std::vector<std::wstring> m_lFrameNames;
	CDXUTComboBox* pComboBox = m_FreeMotionUI.GetComboBox( IDC_FRAME );

	m_ActualMesh->GetFrameNames(m_lFrameNames);
	for (WORD i=0; i<m_lFrameNames.size(); ++i)
	{
		int *pData = new int[3];
		pData[0] = pData[1] = pData[2] = 0;
		pComboBox->AddItem( m_lFrameNames[i].c_str(), (void*) pData );
	}
	pComboBox->SetSelectedByIndex(0);



	return S_OK;
}





//--------------------------------------------------------------------------------------
// This callback function will be called immediately after the Direct3D device has been 
// reset, which will happen after a lost device scenario. This is the best location to 
// create D3DPOOL_DEFAULT resources since these resources need to be reloaded whenever 
// the device is lost. Resources created here should be released in the OnLostDevice 
// callback. 
//--------------------------------------------------------------------------------------
HRESULT CApplication::OnResetDevice( IDirect3DDevice9* pd3dDevice, 
							   const D3DSURFACE_DESC* pBackBufferSurfaceDesc )
{
	HRESULT hr;


	if( m_pFont )
		V_RETURN( m_pFont->OnResetDevice() );

	for (int i=0; i<m_iSMCount; ++i)
		m_SkinningMethods[i]->OnResetDevice();

	// Create a sprite to help batch calls when drawing many lines of text
	V_RETURN( D3DXCreateSprite( pd3dDevice, &m_pTextSprite ) );

	// Setup render state
	DWORD dwFillMode = m_bWireframe ? D3DFILL_WIREFRAME : D3DFILL_SOLID;
	pd3dDevice->SetRenderState( D3DRS_FILLMODE,        dwFillMode  );
	pd3dDevice->SetRenderState( D3DRS_LIGHTING,         TRUE );
	pd3dDevice->SetRenderState( D3DRS_DITHERENABLE,     TRUE );
	pd3dDevice->SetRenderState( D3DRS_ZENABLE,          TRUE );
	pd3dDevice->SetRenderState( D3DRS_CULLMODE,         D3DCULL_CCW );
	pd3dDevice->SetRenderState( D3DRS_AMBIENT,          0x33333333 );
	pd3dDevice->SetRenderState( D3DRS_NORMALIZENORMALS, TRUE );
	pd3dDevice->SetSamplerState( 0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR );
	pd3dDevice->SetSamplerState( 0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR );

	// load the indexed vertex shaders
	DWORD dwShaderFlags = 0;
#if defined(DEBUG_VS) || defined(DEBUG_PS)
	dwShaderFlags |= D3DXSHADER_DEBUG|D3DXSHADER_SKIPVALIDATION;
#endif

	// Setup the projection matrix
	float fAspect = (float)pBackBufferSurfaceDesc->Width / (float)pBackBufferSurfaceDesc->Height;
	D3DXMatrixPerspectiveFovLH( &m_matProj, D3DX_PI/4, fAspect,
		m_ActualMesh->GetObjectRadius()/64.0f, m_ActualMesh->GetObjectRadius()*200.0f );

	pd3dDevice->SetTransform( D3DTS_PROJECTION, &m_matProj );

	// Setup the arcball parameters
	m_ArcBall.SetWindow( pBackBufferSurfaceDesc->Width, pBackBufferSurfaceDesc->Height, 0.85f );
	m_ArcBall.SetTranslationRadius( m_ActualMesh->GetObjectRadius() );

	m_HUD.SetLocation( pBackBufferSurfaceDesc->Width-170, 0 );
	m_HUD.SetSize( 170, 170 );
/*	m_SampleUI.SetLocation( 3, 45 );
	m_SampleUI.SetSize( 240, 70 );
*/

	m_AnimationUI.SetLocation( pBackBufferSurfaceDesc->Width-170, 0 );
	m_AnimationUI.SetSize( 240, 70 );

	m_FreeMotionUI.SetLocation( pBackBufferSurfaceDesc->Width-170, 0 );
	m_FreeMotionUI.SetSize( 240, 70 );
	
	for ( int i=0; i<m_iMeshCount; ++i)
	{
		m_Meshes[i]->GetMeshMenu()->OnResetDevice(pd3dDevice,pBackBufferSurfaceDesc);
	}
	

	return S_OK;
}






//--------------------------------------------------------------------------------------
// This callback function will be called once at the beginning of every frame. This is the
// best location for your application to handle updates to the scene, but is not 
// intended to contain actual rendering calls, which should instead be placed in the 
// OnFrameRender callback.  
//--------------------------------------------------------------------------------------
void CApplication::OnFrameMove( IDirect3DDevice9* pd3dDevice, double fTime, float fElapsedTime )
{
	D3DXVECTOR3 vCenter = m_ActualMesh->GetObjectCenter();
	D3DXMATRIX matTrans;


	D3DXMatrixIdentity(&m_matView);
	/*
	D3DXMatrixTranslation( &matTrans, 0, 0, 2*m_ActualMesh->GetObjectRadius() );

	D3DXMatrixMultiply( &m_matView, &matTrans,                          &m_matView);
	*/

	D3DXMatrixMultiply( &m_matView, m_ArcBall.GetTranslationMatrix(),   &m_matView);
	D3DXMatrixMultiply( &m_matView, m_ArcBall.GetRotationMatrix(),      &m_matView);

	D3DXMatrixTranslation( &matTrans, -vCenter.x, -vCenter.y, -vCenter.z );
	D3DXMatrixMultiply( &m_matView, &matTrans,                          &m_matView);


	if (!m_AnimationUI.GetMinimized())
		if (!m_bPaused) m_ActualMesh->AdvanceTime(fElapsedTime);

}





//--------------------------------------------------------------------------------------
// This callback function will be called at the end of every frame to perform all the 
// rendering calls for the scene, and it will also be called if the window needs to be 
// repainted. After this function has returned, the sample framework will call 
// IDirect3DDevice9::Present to display the contents of the next buffer in the swap chain
//--------------------------------------------------------------------------------------
void CApplication::OnFrameRender( IDirect3DDevice9* pd3dDevice, double fTime, float fElapsedTime )
{
	HRESULT hr;

	// Clear the backbuffer
/*	pd3dDevice->Clear( 0L, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER,
		D3DCOLOR_ARGB(0, 66, 75, 121), 1.0f, 0L );
*/
	pd3dDevice->Clear( 0L, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER,
		m_bgColor, 1.0f, 0L );

	// Setup the light
	D3DXVECTOR3 vLightDir(m_matView._13,m_matView._23,m_matView._33);

	// Setup the light
	D3DLIGHT9 light;
	D3DXVECTOR3 vecLightDirUnnormalized(0.0f, -1.0f, 1.0f);
	ZeroMemory( &light, sizeof(D3DLIGHT9) );
	light.Type        = D3DLIGHT_DIRECTIONAL;
	light.Diffuse.r   = 1.0f;
	light.Diffuse.g   = 1.0f;
	light.Diffuse.b   = 1.0f;
	light.Direction = vLightDir;
	light.Position.x   = 0.0f;
	light.Position.y   = -1.0f;
	light.Position.z   = 1.0f;
	light.Range        = 1000.0f;
	V( pd3dDevice->SetLight(0, &light ) );
	V( pd3dDevice->LightEnable(0, TRUE ) );

	D3DXMATRIXA16 mat, matTrans;
	//D3DXMatrixMultiply(&mat,&m_matView,&m_matProj);


	// Begin the scene
	if( SUCCEEDED( pd3dDevice->BeginScene() ) )
	{

		m_NumVertices = 0;
		m_NumFaces = 0;

		INT iData = (INT)(m_HUD.GetComboBox(IDC_INSTANCES)->GetSelectedData());
		FLOAT fOffset = (iData-1)/2.0f *1.5f*m_ActualMesh->GetObjectRadius();

		for (int i=0; i<iData; ++i) 
			for (int j=0; j<iData; ++j)
			{

				D3DXMatrixTranslation( &matTrans, j*1.5f*m_ActualMesh->GetObjectRadius() - fOffset, 
							i*1.5f*m_ActualMesh->GetObjectRadius() - fOffset, iData*2*m_ActualMesh->GetObjectRadius() );

				D3DXMatrixMultiply( &mat, &m_matView, &matTrans);
     
				pd3dDevice->SetTransform( D3DTS_VIEW,  &mat );

				D3DXMatrixMultiply(&mat,&mat,&m_matProj);
				m_ActualSM->SetMatViewProj(mat);
				m_ActualSM->SetLightDir(vLightDir);

				m_ActualMesh->Render(pd3dDevice);

				m_NumVertices += m_ActualMesh->GetNumVertices();
				m_NumFaces += m_ActualMesh->GetNumFaces();
			}

		RenderText();

		V( m_HUD.OnRender( fElapsedTime ) );
		V( m_AnimationUI.OnRender( fElapsedTime ) );
		V( m_FreeMotionUI.OnRender( fElapsedTime ) );

		V( m_ActualMesh->GetMeshMenu()->GetDialog()->OnRender( fElapsedTime ) );

		// End the scene.
		pd3dDevice->EndScene();
	}
}



//--------------------------------------------------------------------------------------
// Render the help and statistics text. This function uses the ID3DXFont interface for 
// efficient text rendering.
//--------------------------------------------------------------------------------------
void CApplication::RenderText()
{
	// The helper object simply helps keep track of text position, and color
	// and then it calls pFont->DrawText( m_pSprite, strMsg, -1, &rc, DT_NOCLIP, m_clr );
	// If NULL is passed in as the sprite object, then it will work however the 
	// pFont->DrawText() will not be batched together.  Batching calls will improves performance.
	const D3DSURFACE_DESC* pd3dsdBackBuffer = DXUTGetBackBufferSurfaceDesc();
	CDXUTTextHelper txtHelper( m_pFont, m_pTextSprite, 15 );

	// Output statistics
	txtHelper.Begin();
	txtHelper.SetInsertionPos( 5, 5 );
	txtHelper.SetForegroundColor( D3DXCOLOR( 1.0f, 1.0f, 0.0f, 1.0f ) );
	txtHelper.DrawTextLine( DXUTGetFrameStats() );
	txtHelper.DrawTextLine( DXUTGetDeviceStats() );

	txtHelper.SetForegroundColor( D3DXCOLOR( 1.0f, 1.0f, 1.0f, 1.0f ) );
	// Output statistics

	WCHAR pom[256];
	wsprintf(pom,L"Vertices: %d, Triangles: %d, Bones per vertex: %d", m_NumVertices, m_NumFaces, m_ActualMesh->GetNumBonesPerVertex());

	txtHelper.DrawTextLine(pom);
	txtHelper.DrawTextLine(m_ActualMesh->GetErrMessage().c_str());


	// Draw help
	if( m_bShowHelp )
	{
		txtHelper.SetInsertionPos( 10, pd3dsdBackBuffer->Height-15*6 );
		txtHelper.SetForegroundColor( D3DXCOLOR( 1.0f, 0.75f, 0.0f, 1.0f ) );
		txtHelper.DrawTextLine( L"Controls (F10 to hide):" );

		txtHelper.SetInsertionPos( 40, pd3dsdBackBuffer->Height-15*5 );
		txtHelper.DrawTextLine( L"Rotate model: Left click drag\n"
			L"Zoom: Middle click drag\n"
			L"Pane: Right click drag\n"
			L"Quit: ESC" );
	}
	else
	{
		txtHelper.SetInsertionPos( 10, pd3dsdBackBuffer->Height-15*2 );
		txtHelper.SetForegroundColor( D3DXCOLOR( 1.0f, 1.0f, 1.0f, 1.0f ) );
		txtHelper.DrawTextLine( L"Press F10 for help" );
	}

	// If software vp is required and we are using a hwvp device,
	// the mesh is not being displayed and we output an error message here.
	if( m_bUseSoftwareVP && ( m_dwBehaviorFlags & D3DCREATE_HARDWARE_VERTEXPROCESSING ) )
	{
		txtHelper.SetInsertionPos( 5, 85 );
		txtHelper.SetForegroundColor( D3DXCOLOR( 1.0f, 0.0f, 0.0f, 1.0f ) );
		txtHelper.DrawTextLine( L"The HWVP device does not support this skinning method.\n"
			L"Select another skinning method or switch to mixed or software VP." );
	}

	txtHelper.End();
}







//--------------------------------------------------------------------------------------
// Before handling window messages, the sample framework passes incoming windows 
// messages to the application through this callback function. If the application sets 
// *pbNoFurtherProcessing to TRUE, then the sample framework will not process this message.
//--------------------------------------------------------------------------------------
LRESULT CApplication::MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, bool* pbNoFurtherProcessing )
{
	// Give the dialogs a chance to handle the message first
	*pbNoFurtherProcessing = m_HUD.MsgProc( hWnd, uMsg, wParam, lParam );
	if( *pbNoFurtherProcessing )
		return 0;

	*pbNoFurtherProcessing = m_AnimationUI.MsgProc( hWnd, uMsg, wParam, lParam );
	if( *pbNoFurtherProcessing )
		return 0;

	*pbNoFurtherProcessing = m_FreeMotionUI.MsgProc( hWnd, uMsg, wParam, lParam );
	if( *pbNoFurtherProcessing )
		return 0;

	*pbNoFurtherProcessing = m_ActualMesh->GetMeshMenu()->GetDialog()->MsgProc( hWnd, uMsg, wParam, lParam );
	if( *pbNoFurtherProcessing )
		return 0;

	// Pass all remaining windows messages to arcball so it can respond to user input
	m_ArcBall.HandleMessages( hWnd, uMsg, wParam, lParam );

	return 0;
}








//--------------------------------------------------------------------------------------
// As a convenience, the sample framework inspects the incoming windows messages for
// keystroke messages and decodes the message parameters to pass relevant keyboard
// messages to the application.  The framework does not remove the underlying keystroke 
// messages, which are still passed to the application's MsgProc callback.
//--------------------------------------------------------------------------------------
void CApplication::KeyboardProc( UINT nChar, bool bKeyDown, bool bAltDown )
{
	if( bKeyDown )
	{
		switch( nChar )
		{
		case VK_F10: m_bShowHelp = !m_bShowHelp; break;
		}
	}
}










//--------------------------------------------------------------------------------------
// This callback function will be called immediately after the Direct3D device has 
// entered a lost state and before IDirect3DDevice9::Reset is called. Resources created
// in the OnResetDevice callback should be released here, which generally includes all 
// D3DPOOL_DEFAULT resources. See the "Lost Devices" section of the documentation for 
// information about lost devices.
//--------------------------------------------------------------------------------------
void CApplication::OnLostDevice()
{
	if( m_pFont )
		m_pFont->OnLostDevice();

	for(int i=0; i<m_iSMCount; ++i)
		m_SkinningMethods[i]->OnLostDevice();

	SAFE_RELEASE( m_pTextSprite );

}




//--------------------------------------------------------------------------------------
// This callback function will be called immediately after the Direct3D device has 
// been destroyed, which generally happens as a result of application termination or 
// windowed/full screen toggles. Resources created in the OnCreateDevice callback 
// should be released here, which generally includes all D3DPOOL_MANAGED resources. 
//--------------------------------------------------------------------------------------
void CApplication::OnDestroyDevice()
{
	for(int i=0; i<m_iSMCount; ++i)
		m_SkinningMethods[i]->OnDestroyDevice();

	for(int i=0; i<m_iMeshCount; ++i)
		m_Meshes[i]->OnDestroyDevice();

	SAFE_RELEASE( m_pFont );

	CDXUTComboBox* pComboBox = m_FreeMotionUI.GetComboBox( IDC_FRAME );
	for (WORD i=0; i<pComboBox->GetNumItems(); ++i)
		delete[] pComboBox->GetItem(i)->pData;

	pComboBox->RemoveAllItems();


}




void CApplication::InitUserInterface()
{
	// Initialize dialogs
	m_HUD.SetCallback( OnGUIEvent ); int iY = 10; 
	m_HUD.AddButton( IDC_TOGGLEFULLSCREEN, L"Toggle full screen (F1)", 35, iY, 125, 22, VK_F1 );
	m_HUD.AddButton( IDC_CHANGEDEVICE, L"Change device (F2)", 35, iY += 24, 125, 22, VK_F2 );
	m_HUD.AddButton( IDC_FILLMODE, L"Solid (F4)",35, iY += 24, 125, 22, VK_F4 );





	// Typy metod
	m_HUD.AddComboBox( IDC_METHOD, -100, iY += 24, 260, 24, L'M' );

	for (WORD i=0; i<m_iSMCount; ++i)
		m_HUD.GetComboBox( IDC_METHOD )->AddItem( m_SkinningMethods[i]->GetLabel(), (void*)i );

	// Pocet instanci
	m_HUD.AddComboBox( IDC_INSTANCES, -100, iY += 24, 260, 24, L'M' );
	m_HUD.GetComboBox( IDC_INSTANCES )->AddItem( L"1 mesh", (void*)1 );
	m_HUD.GetComboBox( IDC_INSTANCES )->AddItem( L"4 meshes", (void*)2 );
	m_HUD.GetComboBox( IDC_INSTANCES )->AddItem( L"16 meshes", (void*)4 );
	m_HUD.GetComboBox( IDC_INSTANCES )->AddItem( L"36 meshes", (void*)6 );
	m_HUD.GetComboBox( IDC_INSTANCES )->AddItem( L"64 meshes", (void*)8 );
	m_HUD.GetComboBox( IDC_INSTANCES )->AddItem( L"100 meshes", (void*)10 );
	m_HUD.GetComboBox( IDC_INSTANCES )->AddItem( L"225 meshes", (void*)15 );


	// Tlacitko Free motion/Animation
	m_HUD.AddButton( IDC_FREE_ANIMATION, L"Animation (F5)",35, iY += 6*24, 125, 22, VK_F5 );

	iY += 24;

	int iYPom = iY;

	// UI Animace
	m_AnimationUI.SetCallback( OnGUIEvent ); 
	m_AnimationUI.AddButton( IDC_PAUSE, L"Go / Pause (F6)",35, iY += 24, 125, 22, VK_F6 );




	// Animation
	// Jmeno animace
	// Go/Pause

	// Free motion
	// Reset
	// Yaw, Pitch, Roll
	iY = iYPom;

	// UI Free Motion
	m_FreeMotionUI.SetCallback( OnGUIEvent ); 
	m_FreeMotionUI.AddSlider(IDC_YAW,-70,iY += 24,220,22,0,100,0);
	m_FreeMotionUI.AddSlider(IDC_PITCH,-70,iY += 24,220,22,0,100,0);
	m_FreeMotionUI.AddSlider(IDC_ROLL,-70,iY += 24,220,22,0,100,0);
	CDXUTSlider* pSlider = m_FreeMotionUI.GetSlider(IDC_YAW);

	// Kosti
	m_FreeMotionUI.AddComboBox( IDC_FRAME, -70,iY += 24, 230, 24, L'M' );

	m_FreeMotionUI.SetMinimized(true);

	for (int i=0; i<m_iMeshCount; ++i)
	{
		m_Meshes[i]->GetMeshMenu()->InitUserInterface();
		m_Meshes[i]->GetMeshMenu()->GetDialog()->SetCallback( OnGUIEvent );
		m_Meshes[i]->GetMeshMenu()->GetDialog()->SetMinimized(true);
	}

}
