/*! \file 
    \brief Zkladn tda aplikace
*/ 

#pragma once

#include <vector>
#include <string>

class CSkinningMethod;
class CVirtualMesh;


#define MAX_SKINNING_METHODS 20

/// Zkladn tda aplikace
class CApplication
{
public:

    /// Filtrace podporovanch D3D zazen podle parametr
	static bool		CALLBACK	IsDeviceAcceptable( D3DCAPS9* pCaps, D3DFORMAT AdapterFormat, D3DFORMAT BackBufferFormat, bool bWindowed );
    /// Nastaven nkterch parametr D3D zazen (hlavn co se te shader a debugovn)
	static void		CALLBACK	ModifyDeviceSettings( DXUTDeviceSettings* pDeviceSettings, const D3DCAPS9* pCaps );
    /// Reakce na vytvoen D3D zazen
	static HRESULT	CALLBACK	OnCreateDevice( IDirect3DDevice9* pd3dDevice, const D3DSURFACE_DESC* pBackBufferSurfaceDesc );
    /// Reakce na zmnu D3D zazen
	static HRESULT	CALLBACK	OnResetDevice( IDirect3DDevice9* pd3dDevice, const D3DSURFACE_DESC* pBackBufferSurfaceDesc );
    /// Reakce na zmnu D3D zazen
	static void		CALLBACK	OnLostDevice();
    /// Reakce na zruen D3D zazen
	static void		CALLBACK	OnDestroyDevice();
    /// Reakce na posun simulovanho asu
	static void		CALLBACK	OnFrameMove( IDirect3DDevice9* pd3dDevice, double fTime, float fElapsedTime );
    /// Render scny
	static void		CALLBACK	OnFrameRender( IDirect3DDevice9* pd3dDevice, double fTime, float fElapsedTime );
    /// Reakce na window messages
	static LRESULT	CALLBACK	MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, bool* pbNoFurtherProcessing );
    /// Reakce na zmknut klvesy
	static void		CALLBACK	KeyboardProc( UINT nChar, bool bKeyDown, bool bAltDown  );
    /// Reakce na zmknut prvku uivatelskho rozhran
	static void		CALLBACK	OnGUIEvent( UINT nEvent, int nControlID, CDXUTControl* pControl );
    /// Registrace metod skinningu a model do aplikace
	static void		RegisterMethods(CSkinningMethod** methods, CVirtualMesh** meshes);
    /// Nastaven barvy pozad
	static void		SetBackgroundColor(D3DCOLOR color) {m_bgColor = color; }
    /// Sputn aplikace
	static HRESULT	Run();
    /// Analza pkazovho dku
	static void		ParseCommandLine(std::vector<std::wstring> &commands, LPWSTR strCmdLine);


protected:

	static void SetCallbackFunctions();
	static void InitUserInterface();
	static void RenderText();

	static void RetrieveFrameNames();

	static void YPRChange();
	static void FrameChange(LPCSTR FrameNamem, int *pData);
	static void FreeAnimationButtonEvent();
	static void PoseSet();

	
	enum IDControl { 
		IDC_TOGGLEFULLSCREEN = 1000,
		IDC_CHANGEDEVICE,
		IDC_FILLMODE,
		IDC_METHOD,
		IDC_INSTANCES,
		IDC_FREE_ANIMATION,

		IDC_PAUSE,
		IDC_ANIMATION,

		IDC_FRAME,
		IDC_YAW,
		IDC_PITCH,
		IDC_ROLL
	};


protected:

	// Vsechny skinning metody, ke kazde SM je prirazeny nejaky typ modelu
	static CSkinningMethod*				m_SkinningMethods[MAX_SKINNING_METHODS];
	static CSkinningMethod*				m_ActualSM;
	static int							m_iSMCount;

	static D3DCOLOR						m_bgColor;

	// Typy modelu
	static CVirtualMesh*				m_Meshes[MAX_SKINNING_METHODS];
	static CVirtualMesh*				m_ActualMesh;
	static int							m_iMeshCount;

	static int							m_NumVertices;
	static int							m_NumFaces;

	static CD3DArcBall					m_ArcBall;				// Arcball for camera control

	static bool							m_bPaused;
	static bool							m_bWireframe;
	static bool							m_bShowHelp;

	static CDXUTDialog					m_HUD;                  // dialog for standard controls
	static CDXUTDialog					m_AnimationUI;			
	static CDXUTDialog					m_FreeMotionUI;

	static ID3DXFont*					m_pFont;				// Font for drawing text
	static ID3DXSprite*					m_pTextSprite;			// Sprite for batching draw text calls

	static D3DXMATRIXA16				m_matView;              // View matrix
	static D3DXMATRIXA16				m_matProj;              // Projection matrix


	static DWORD						m_dwBehaviorFlags;      // Behavior flags of the 3D device
	static bool							m_bUseSoftwareVP;		// Flag to indicate whether software vp is
																// required due to lack of hardware
	
	static IDirect3DDevice9*				m_pd3dDevice;


};