#include "dxstdafx.h"

#include "CApplication.h"
#include "CSkinningMethod.h"
#include "CMesh.h"
#include "CMeshMenu.h"


//--------------------------------------------------------------------------------------
// Handles the GUI events
//--------------------------------------------------------------------------------------
void CApplication::OnGUIEvent( UINT nEvent, int nControlID, CDXUTControl* pControl )
{
	switch( nControlID )
	{
	case IDC_TOGGLEFULLSCREEN:	DXUTToggleFullScreen(); break;
	case IDC_CHANGEDEVICE:		DXUTSetShowSettingsDialog( !DXUTGetShowSettingsDialog() ); break;
	case IDC_PAUSE:				m_bPaused = !m_bPaused; break;
	case IDC_FILLMODE:			
		{
			CDXUTButton *pButton = (CDXUTButton *) pControl;
			m_bWireframe = !m_bWireframe; 
			if ( m_bWireframe)	pButton->SetText(L"Wireframe (F4)");
			else pButton->SetText(L"Solid (F4)");

			DWORD dwFillMode = m_bWireframe ? D3DFILL_WIREFRAME : D3DFILL_SOLID;
			m_pd3dDevice->SetRenderState( D3DRS_FILLMODE,        dwFillMode  );
			break;
		}

	case IDC_METHOD:
		{
			int iNew = (int)(size_t)((CDXUTComboBox*)pControl)->GetSelectedData();

			// If the selected skinning method is different than the current one
			if( m_SkinningMethods[iNew] != m_ActualSM)
			{
				if( m_SkinningMethods[iNew]->GetMeshID() != m_ActualSM->GetMeshID() )
				{
					double dTime = m_ActualMesh->GetAnimationTime();
					int iMeshID = m_SkinningMethods[iNew]->GetMeshID();
					
					m_ActualMesh->GetMeshMenu()->GetDialog()->SetMinimized(true);

					m_ActualMesh = m_Meshes[iMeshID];

					dTime -= m_ActualMesh->GetAnimationTime();

					m_ActualMesh->AdvanceTime(dTime);
				}

				if (!m_FreeMotionUI.GetMinimized())
					PoseSet();

				m_ActualSM = m_SkinningMethods[iNew];
				m_ActualMesh->SetSkinningMethod(m_ActualSM);

				m_ActualMesh->GetMeshMenu()->GetDialog()->SetMinimized(m_FreeMotionUI.GetMinimized());


			}

			break;
		}

	case IDC_FREE_ANIMATION:
		{
			FreeAnimationButtonEvent();
			break;
		}

	case IDC_FRAME:
		{
			// vyber kosti pro pohyb
			DXUTComboBoxItem* pItem =  ((CDXUTComboBox*)pControl)->GetSelectedItem();
			CHAR pom[256];
			wcstombs(pom,pItem->strText,255);
			FrameChange(pom, (int*)pItem->pData);
			break;
		}

	case IDC_YAW:
	case IDC_PITCH:
	case IDC_ROLL:
		{
			YPRChange();
			break;
		}
	default:
		m_ActualMesh->GetMeshMenu()->OnGuiEvent(nEvent, nControlID, pControl);
	}

}


void CApplication::YPRChange()
{
	float fYaw = 2.0f * D3DX_PI * m_FreeMotionUI.GetSlider(IDC_YAW)->GetValue()/100.0f;
	float fPitch = 2.0f * D3DX_PI * m_FreeMotionUI.GetSlider(IDC_PITCH)->GetValue()/100.0f;
	float fRoll = 2.0f * D3DX_PI * m_FreeMotionUI.GetSlider(IDC_ROLL)->GetValue()/100.0f;

	int *pData = (int*) m_FreeMotionUI.GetComboBox(IDC_FRAME)->GetSelectedData();
	pData[0] = m_FreeMotionUI.GetSlider(IDC_YAW)->GetValue();
	pData[1] = m_FreeMotionUI.GetSlider(IDC_PITCH)->GetValue();
	pData[2] = m_FreeMotionUI.GetSlider(IDC_ROLL)->GetValue();

	D3DXMATRIX mat;
	D3DXMatrixRotationYawPitchRoll(&mat,fYaw,fPitch,fRoll);
	m_ActualMesh->TransformSelectedFrame(&mat);

}



void CApplication::FrameChange(LPCSTR FrameName, int *pData)
{
	m_ActualMesh->SelectFrame(FrameName);

	m_FreeMotionUI.GetSlider(IDC_YAW)->SetValue(pData[0]);
	m_FreeMotionUI.GetSlider(IDC_PITCH)->SetValue(pData[1]);
	m_FreeMotionUI.GetSlider(IDC_ROLL)->SetValue(pData[2]);

}



void CApplication::FreeAnimationButtonEvent()
{
	CDXUTButton *pButton = m_HUD.GetButton(IDC_FREE_ANIMATION);
	if (m_FreeMotionUI.GetMinimized()) 
	{
		m_FreeMotionUI.SetMinimized(false);
		m_ActualMesh->GetMeshMenu()->GetDialog()->SetMinimized(false);
		m_AnimationUI.SetMinimized(true);
		pButton->SetText(L"Free Motion (F5)");

		// nastaveni pozy
		PoseSet();

	} else
	{
		m_FreeMotionUI.SetMinimized(true);
		m_ActualMesh->GetMeshMenu()->GetDialog()->SetMinimized(true);
		m_AnimationUI.SetMinimized(false);
		pButton->SetText(L"Animation (F5)");
		m_ActualMesh->AdvanceTime(0.0);
	}

}



void CApplication::PoseSet()
{
	CDXUTComboBox* pComboBox = m_FreeMotionUI.GetComboBox( IDC_FRAME );
	DXUTComboBoxItem* pItem;
	CHAR pom[256];
	for (WORD i=0; i<pComboBox->GetNumItems(); ++i)
	{
		pItem = pComboBox->GetItem(i);
		wcstombs(pom,pItem->strText,255);
		m_ActualMesh->SelectFrame(pom);

		float fYaw = 2.0f * D3DX_PI * ((int*)pItem->pData)[0]/100.0f;
		float fPitch = 2.0f * D3DX_PI * ((int*)pItem->pData)[1]/100.0f;
		float fRoll = 2.0f * D3DX_PI * ((int*)pItem->pData)[2]/100.0f;

		D3DXMATRIX mat;
		D3DXMatrixRotationYawPitchRoll(&mat,fYaw,fPitch,fRoll);
		m_ActualMesh->TransformSelectedFrame(&mat);
	}
	pItem = pComboBox->GetSelectedItem();
	wcstombs(pom,pItem->strText,255);
	m_ActualMesh->SelectFrame(pom);

}