#include "dxstdafx.h"

#include "CRBFInterp.h"


CRBFInterp::CRBFInterp( INT count, D3DXVECTOR3 *pExamples, FLOAT* pSigmas)
{
	m_iCount = count;
	m_pExamples = new D3DXVECTOR3[m_iCount];
	m_pMatrixLambda = new matrix<double>(m_iCount,m_iCount);
	m_pSigmas = new FLOAT[m_iCount];

	matrix<double> &mat = *m_pMatrixLambda;

	for (int i=0; i<m_iCount; ++i)
	{
		m_pExamples[i] = pExamples[i];
		m_pSigmas[i] = pSigmas[i];
	}

	for (int i=0; i<m_iCount; ++i)
		for (int j=0; j<m_iCount; ++j)
			mat(i,j) = RBF(i,Length(pExamples[i],pExamples[j]));

	mat = mat.inv();

}




CRBFInterp::~CRBFInterp()
{
	delete m_pMatrixLambda;
	delete[] m_pExamples;
	delete[] m_pSigmas;
}




double CRBFInterp::RBF(INT i, double fDist)
{
	// vypocist RBF
	return exp(-fDist*fDist/(m_pSigmas[i]*m_pSigmas[i]) );
}


double CRBFInterp::Length(D3DXVECTOR3 &vec1, D3DXVECTOR3 &vec2)
{
	// vypocist vzdalenost
	double fCos = D3DXVec3Dot(&vec1,&vec2);
	if (fCos > 1.0f) fCos = .99999f; else
	if (fCos < -1.0f) fCos = -0.99999;
	
	return acos(fCos);
}



void CRBFInterp::Interpolate(D3DXVECTOR3 vec, FLOAT* pWeights)
{
	double dWeight;
	matrix<double> &mat = *m_pMatrixLambda;

	for (int i=0; i<m_iCount; ++i)
	{
		dWeight = 0.0f;
		for (int j=0; j<m_iCount; ++j)
			dWeight += mat(i,j) * RBF(j,Length(vec, m_pExamples[j]));

		pWeights[i] = (FLOAT) dWeight;
	}

}
