/*! \file 
    \brief Obecn nahrn modelu ze souboru.
*/ 

#pragma once


class CAllocationMeshCB;

/// Zkladn mesh kontejner
struct D3DXMESHCONTAINER_BASE: public D3DXMESHCONTAINER
{
	/// Pole textur
	LPDIRECT3DTEXTURE9*  ppTextures;       

	// SkinMesh info             
	LPD3DXMESH           pOrigMesh;
	LPD3DXATTRIBUTERANGE pAttributeTable;
	DWORD                NumAttributeGroups; 
	DWORD                NumInfl;
	LPD3DXBUFFER         pBoneCombinationBuf;
	D3DXMATRIX*          pBoneOffsetMatrices;
	D3DXQUATERNION*		 pBoneOffsetQuats;
	D3DXVECTOR3*		 pBoneOffsetTrans;
	DWORD                NumPaletteEntries;
	bool				 UseSoftwareVP;

	D3DXMESHCONTAINER_BASE()
	{
		ppTextures = NULL;       		        
		pOrigMesh = NULL;
		pAttributeTable = NULL;
		NumAttributeGroups = NULL;
		NumInfl = NULL;
		pBoneCombinationBuf = NULL;
		pBoneOffsetMatrices = NULL;
		pBoneOffsetQuats = NULL;
		pBoneOffsetTrans = NULL;

		NumPaletteEntries = NULL;
	
		Name = NULL;

		MeshData.pMesh = NULL;

		pMaterials = NULL;
		pEffects = NULL;
		NumMaterials = 0;
		pAdjacency = NULL;
		pSkinInfo = NULL;

		UseSoftwareVP = false;

	}

	virtual ~D3DXMESHCONTAINER_BASE()
	{
		SAFE_DELETE_ARRAY( Name );
		SAFE_DELETE_ARRAY( pAdjacency );
		SAFE_DELETE_ARRAY( pMaterials );
		SAFE_DELETE_ARRAY( pBoneOffsetMatrices );
		SAFE_DELETE_ARRAY( pBoneOffsetQuats );
		SAFE_DELETE_ARRAY( pBoneOffsetTrans );

		// release all the allocated textures
		if (ppTextures != NULL)
		{
			for (int iMaterial = 0; iMaterial < NumMaterials; iMaterial++)
			{
				SAFE_RELEASE( ppTextures[iMaterial] );
			}
		}

		SAFE_DELETE_ARRAY( ppTextures );
		SAFE_RELEASE( pBoneCombinationBuf );
		SAFE_RELEASE( MeshData.pMesh );
		SAFE_RELEASE( pSkinInfo );
		SAFE_RELEASE( pOrigMesh );	

	}
};


/// Zkladn struktura kosti
struct D3DXFRAME_BASE : public D3DXFRAME
{
};

/// Tda zptnch voln pro nahrn modelu ze souboru
class CAllocateHierarchy: public ID3DXAllocateHierarchy
{
public:
    
    /// Zptn voln pro vytvoen struktury kosti
	STDMETHOD(CreateFrame)(THIS_ LPCSTR Name, LPD3DXFRAME *ppNewFrame);

    /// Zptn voln pro vytvoen mesh kontejneru, kter uchovv data modelu
	STDMETHOD(CreateMeshContainer)(THIS_ 
		LPCSTR Name, 
		CONST D3DXMESHDATA *pMeshData,
		CONST D3DXMATERIAL *pMaterials, 
		CONST D3DXEFFECTINSTANCE *pEffectInstances, 
		DWORD NumMaterials, 
		CONST DWORD *pAdjacency, 
		LPD3DXSKININFO pSkinInfo, 
		LPD3DXMESHCONTAINER *ppNewMeshContainer);
	STDMETHOD(DestroyFrame)(THIS_ LPD3DXFRAME pFrameToFree);
	STDMETHOD(DestroyMeshContainer)(THIS_ LPD3DXMESHCONTAINER pMeshContainerBase);

	CAllocateHierarchy(CAllocationMeshCB *pMesh) 
		{ m_pMesh = pMesh; m_NumBoneMatricesMax = 0; }

	virtual ~CAllocateHierarchy() {};

protected:

	virtual HRESULT GenerateSkinnedMesh( IDirect3DDevice9 *pd3dDevice, D3DXMESHCONTAINER_BASE *pMeshContainer );

	CAllocationMeshCB*	m_pMesh;
	int					m_NumBoneMatricesMax;
};
