#include "dxstdafx.h"

#include "CAllocateHierarchyFixed.h"
#include "CVirtualMesh.h"




//--------------------------------------------------------------------------------------
// This function uses the pSkinInfo of the mesh 
// container to generate the desired drawable mesh and bone combination 
// table.
//--------------------------------------------------------------------------------------
HRESULT CAllocateHierarchyFixed::GenerateSkinnedMesh( IDirect3DDevice9 *pd3dDevice, D3DXMESHCONTAINER_BASE *pMeshContainer )
{
	HRESULT hr = S_OK;
	D3DCAPS9 d3dCaps;

	pd3dDevice->GetDeviceCaps( &d3dCaps );


	if( pMeshContainer->pSkinInfo == NULL )
		return hr;



	SAFE_RELEASE( pMeshContainer->MeshData.pMesh );
	SAFE_RELEASE( pMeshContainer->pBoneCombinationBuf );



	hr = pMeshContainer->pSkinInfo->ConvertToBlendedMesh
		(
		pMeshContainer->pOrigMesh,
		D3DXMESH_MANAGED, 
		pMeshContainer->pAdjacency, 
		NULL, NULL, NULL, 
		&pMeshContainer->NumInfl,
		&pMeshContainer->NumAttributeGroups, 
		&pMeshContainer->pBoneCombinationBuf, 
		&pMeshContainer->MeshData.pMesh
		);


	// If the device can only do 2 matrix blends, ConvertToBlendedMesh cannot approximate all meshes to it
	// Thus we split the mesh in two parts: The part that uses at most 2 matrices and the rest. The first is
	// drawn using the device's HW vertex processing and the rest is drawn using SW vertex processing.
	LPD3DXBONECOMBINATION rgBoneCombinations  = reinterpret_cast<LPD3DXBONECOMBINATION>(pMeshContainer->pBoneCombinationBuf->GetBufferPointer());

	// look for any set of bone combinations that do not fit the caps
	for (UINT iAttrib = 0; iAttrib < pMeshContainer->NumAttributeGroups; iAttrib++)
	{
		DWORD cInfl   = 0;

		for (DWORD iInfl = 0; iInfl < pMeshContainer->NumInfl; iInfl++)
		{
			if (rgBoneCombinations[iAttrib].BoneId[iInfl] != UINT_MAX)
			{
				++cInfl;
			}
		}

		if (cInfl > d3dCaps.MaxVertexBlendMatrices)
		{
			pMeshContainer->UseSoftwareVP = true;
			break;

		}
	}
	return hr;
}
