#include "dxstdafx.h"

#include "CAllocateHierarchyFixedIndexed.h"
#include "CVirtualMesh.h"




//--------------------------------------------------------------------------------------
// This function uses the pSkinInfo of the mesh 
// container to generate the desired drawable mesh and bone combination 
// table.
//--------------------------------------------------------------------------------------
HRESULT CAllocateHierarchyFixedIndexed::GenerateSkinnedMesh( IDirect3DDevice9 *pd3dDevice, D3DXMESHCONTAINER_BASE *pMeshContainer )
{
	HRESULT hr = S_OK;
	D3DCAPS9 d3dCaps;
	pd3dDevice->GetDeviceCaps( &d3dCaps );

	if( pMeshContainer->pSkinInfo == NULL )
		return hr;



	SAFE_RELEASE( pMeshContainer->MeshData.pMesh );
	SAFE_RELEASE( pMeshContainer->pBoneCombinationBuf );


	DWORD NumMaxFaceInfl;
	DWORD Flags = 0;

	LPDIRECT3DINDEXBUFFER9 pIB;
	hr = pMeshContainer->pOrigMesh->GetIndexBuffer(&pIB);
	if (FAILED(hr))
		goto e_Exit;

	hr = pMeshContainer->pSkinInfo->GetMaxFaceInfluences(pIB, pMeshContainer->pOrigMesh->GetNumFaces(), &NumMaxFaceInfl);
	pIB->Release();
	if (FAILED(hr))
		goto e_Exit;

	// 12 entry palette guarantees that any triangle (4 independent influences per vertex of a tri)
	// can be handled
	NumMaxFaceInfl = min(NumMaxFaceInfl, 12);

	if( d3dCaps.MaxVertexBlendMatrixIndex + 1 < NumMaxFaceInfl )
	{
		// HW does not support indexed vertex blending. Use SW instead
		pMeshContainer->NumPaletteEntries = min(256, pMeshContainer->pSkinInfo->GetNumBones());
        pMeshContainer->UseSoftwareVP = true;
		Flags |= D3DXMESH_SYSTEMMEM;
	}
	else
	{
		// using hardware - determine palette size from caps and number of bones
		// If normals are present in the vertex data that needs to be blended for lighting, then 
		// the number of matrices is half the number specified by MaxVertexBlendMatrixIndex.
		pMeshContainer->NumPaletteEntries = min( ( d3dCaps.MaxVertexBlendMatrixIndex + 1 ) / 2, 
			pMeshContainer->pSkinInfo->GetNumBones() );
        pMeshContainer->UseSoftwareVP = false;
		Flags |= D3DXMESH_MANAGED;
	}

	hr = pMeshContainer->pSkinInfo->ConvertToIndexedBlendedMesh
		(
		pMeshContainer->pOrigMesh,
		Flags, 
		pMeshContainer->NumPaletteEntries, 
		pMeshContainer->pAdjacency, 
		NULL, NULL, NULL, 
		&pMeshContainer->NumInfl,
		&pMeshContainer->NumAttributeGroups, 
		&pMeshContainer->pBoneCombinationBuf, 
		&pMeshContainer->MeshData.pMesh);
	if (FAILED(hr))
		goto e_Exit;

e_Exit:
	return hr;
}
