#include "dxstdafx.h"

#include "CExpMesh.h"
#include "CQuatMesh.h"
#include "CSkinningMethod.h"
#include "VertexDecl.h"
#include "CRBFInterp.h"
#include "CConfigFile.h"
#include "CExpMenu.h"
#include <fstream>


#define EPSILON 0.00001

bool QuaternionEquals(D3DXQUATERNION *q1, D3DXQUATERNION *q2)
{
	if ( abs( q1->x - q2->x ) < EPSILON &&
		 abs( q1->y - q2->y ) < EPSILON &&
		 abs( q1->z - q2->z ) < EPSILON &&
		 abs( q1->w - q2->w ) < EPSILON ) return true;

	return false;
}






CExpMesh::CExpMesh(std::wstring filename, bool weld, bool optimize) : 
	CQuatMesh(filename, weld, optimize)
{
	
	m_pExampleWeights = NULL;
	m_listMesh.clear();

	delete m_pMeshMenu;
	m_pMeshMenu = new CExpMenu(this);

	m_bShowExVert = false;
	m_iBoneExIndex = -1;

	LoadExamples();

}



CExpMesh::~CExpMesh()
{ 
	for (int i=0; i<m_listMesh.size(); ++i)
	{
		delete m_listMesh[i];
		m_listMesh[i] = NULL;
	}
}



void CExpMesh::LoadExamples()
{
	WCHAR strpom[10];
	std::wstring str;
	int i=1;
	FILE *file;

	while(1)
	{
		wsprintf(strpom,L"%d",i);
		str = std::wstring(m_strFilename,0, m_strFilename.length()-2);
		str += std::wstring(strpom);
		str += L".x";

		if (file = _wfopen(str.c_str(),L"r") ) 
		{
			fclose(file);
			m_listMesh.push_back(new CQuatMesh(str,false,false) );

		}
		else 
			break;
		++i;

	} 

	// load config souboru
	str = std::wstring(m_strFilename,0, m_strFilename.length()-1);
	str += std::wstring(L"cfg");

	m_pConfigFile = new CConfigFile(str);

}


HRESULT	CExpMesh::OnCreateDevice(IDirect3DDevice9* pd3dDevice, DWORD dwBehaviorFlags)
{
	HRESULT hr;

	for (int i=0; i<m_listMesh.size(); ++i)
		V_RETURN(m_listMesh[i]->OnCreateDevice(pd3dDevice, dwBehaviorFlags));


	return CQuatMesh::OnCreateDevice(pd3dDevice, dwBehaviorFlags);
}






void	CExpMesh::OnDestroyDevice()
{
	for (MeshListIt it=m_listMesh.begin(); it!=m_listMesh.end(); it++)
		(*it)->OnDestroyDevice();

	CQuatMesh::OnDestroyDevice();
}



void CExpMesh::AllocForBones(int NumBones)
{

	CQuatMesh::AllocForBones(NumBones);

	m_pExampleWeights = new D3DXVECTOR4[NumBones];
}



void CExpMesh::FreeForBones()
{
	CQuatMesh::FreeForBones();

	delete[] m_pExampleWeights;

	m_pExampleWeights = NULL;
}



void CExpMesh::SetupBonePointersOnMeshAlloc(LPD3DXMESHCONTAINER pMeshContainerBase, int iBones)
{
	CQuatMesh::SetupBonePointersOnMeshAlloc(pMeshContainerBase,iBones);

	D3DXMESHCONTAINER_EXP *pMeshContainer = (D3DXMESHCONTAINER_EXP*)pMeshContainerBase;

	pMeshContainer->ppExampleWeightsPtrs = new D3DXVECTOR4*[iBones];

}

void CExpMesh::SetupBonePointersOnMeshCB(LPD3DXMESHCONTAINER pMeshContainerBase, LPD3DXFRAME pFrameBase, int iBone)
{
	CQuatMesh::SetupBonePointersOnMeshCB(pMeshContainerBase,pFrameBase,iBone);

	D3DXMESHCONTAINER_EXP *pMeshContainer = (D3DXMESHCONTAINER_EXP*)pMeshContainerBase;
	D3DXFRAME_EXP* pFrame = (D3DXFRAME_EXP*) pFrameBase;

	pMeshContainer->ppExampleWeightsPtrs[iBone] = &pFrame->fExampleWeights;

}



void	CExpMesh::UpdateFramesPreCB(LPD3DXFRAME pFrameBase,LPD3DXFRAME pFrameParent)
{
	D3DXFRAME_EXP *pFrame = (D3DXFRAME_EXP*)  pFrameBase;

	FLOAT fLength[MAX_EXAMPLES];


	CQuatMesh::UpdateFramesPreCB(pFrameBase,pFrameParent);

	// vypocitat relativni vzdalenosti quaternionu examplu od aktualniho

	// neni-li nazev, nemuze zasahovat do skinning -> nulove vahy
	if (!pFrameBase->Name || !strcmp(pFrameBase->Name,"") )
	{
		for (int i=0; i<MAX_EXAMPLES; ++i)
			fLength[i] = 0.0f;

		return;
	}



	// Shepard's Interpolation

/*
	D3DXVECTOR3 vecBoneAct, vecBoneEx, vecPom;
	static FLOAT fEpsilon = 0.000000000001f;
	FLOAT fSumLength = 0.0f;
	FLOAT fPomLength, fCos;
	D3DXMATRIX mat;



	vecBoneAct.x = pFrame->TransformationMatrix._11;
	vecBoneAct.y = pFrame->TransformationMatrix._12;
	vecBoneAct.z = pFrame->TransformationMatrix._13;

	vecBoneEx.x = pFrame->matStartingPose._11;
	vecBoneEx.y = pFrame->matStartingPose._12;
	vecBoneEx.z = pFrame->matStartingPose._13;

	vecPom = vecBoneAct - vecBoneEx;

	fCos = D3DXVec3Dot(&vecBoneEx,&vecBoneAct);
	if (fCos > 1.0f) fCos = 1.0f - fEpsilon;
	fPomLength = acos(fCos);
	if (fCos < 0.0f) fCos = 0.0f; else fCos = 1.0f;
	fSumLength += 1.0f / ((fPomLength + fEpsilon) * (fPomLength + fEpsilon));

	for (int i=0; i<MAX_EXAMPLES; ++i)
	{
		if (pFrame->iPoseExampleIndex[i] == -1)
		{
			fLength[i] = 0.0f;
			continue;
		} 

		D3DXMatrixRotationQuaternion(&mat,&pFrame->quatExamples[i]);
		vecBoneEx.x = mat._11;
		vecBoneEx.y = mat._12;
		vecBoneEx.z = mat._13;
		vecPom = vecBoneAct - vecBoneEx;

		fCos = D3DXVec3Dot(&vecBoneEx,&vecBoneAct);
		if (fCos > 1.0f) fCos = 1.0f - fEpsilon;
		fPomLength = acos(fCos);
		if (fCos < 0.0f) fCos = 0.0f; else fCos = 1.0f;
		fLength[i] = 1.0f / ((fPomLength + fEpsilon) * (fPomLength + fEpsilon));
		fSumLength += fLength[i];
	}

	for (int i=0; i<MAX_EXAMPLES; ++i)
	{
		pFrame->fRelLength[i] = fLength[i] / fSumLength;
	}


*/
	


	D3DXVECTOR3 vecBoneAct;

	vecBoneAct.x = pFrame->TransformationMatrix._11;
	vecBoneAct.y = pFrame->TransformationMatrix._12;
	vecBoneAct.z = pFrame->TransformationMatrix._13;

	if (pFrame->pInterp)
		pFrame->pInterp->Interpolate(vecBoneAct,pFrame->fRelLength);

}



void CExpMesh::UpdateFramesPostCB(LPD3DXFRAME pFrameBase,LPD3DXFRAME pFrameParent)
{
	D3DXFRAME_EXP *pFrame = (D3DXFRAME_EXP*)  pFrameBase;

	//zatim jen pro jednu kost
	D3DXFRAME_EXP *pFrameChild = (D3DXFRAME_EXP*) pFrame->pFrameFirstChild;

	while ( pFrameChild && !strcmp(pFrameChild->Name,"") ) 
		pFrameChild = (D3DXFRAME_EXP*) pFrameChild->pFrameSibling;

	if (pFrameChild == NULL) return;

	// posunuti v indexech, protoze prvni(zakladni) poza ma vzdy vahu 1.0f
	for(int i=0; i<MAX_EXAMPLES; ++i)
		pFrame->fExampleWeights[i] =  pFrameChild->fRelLength[i+1];
	//pFrame->fExampleWeights[0] =  1.0f;//pFrameChild->fRelLength[i+1];


}



void CExpMesh::SelectFrame(LPCSTR BoneName)
{
	CQuatMesh::SelectFrame(BoneName);

	((CExpMenu*)m_pMeshMenu)->SetPoseCount(((D3DXFRAME_EXP*)m_pFrameSelected)->iPoseCount);
	
	// najdi kost, jejichz vrcholy se meni podle kosti se jmenem BoneName

	LPD3DXFRAME pParentFrame = FindParent(m_pFrameRoot,BoneName);

	if (!pParentFrame || !pParentFrame->Name)
	{
		m_iBoneExIndex = -1;
		return;
	}

	INT cBones = m_pMeshContainer->pSkinInfo->GetNumBones();

	for (INT iBone = 0; iBone < cBones; iBone++)
	{
		LPCSTR  a = m_pMeshContainer->pSkinInfo->GetBoneName(iBone);
		if (!strcmp(a,pParentFrame->Name))
		{
			m_iBoneExIndex = iBone;
			return;
		}
	}

	m_iBoneExIndex = -1;

}



void CExpMesh::SetPoseToSelectedFrame(int iPose)
{
	D3DXFRAME_EXP*  m_pFrame = (D3DXFRAME_EXP*) m_pFrameSelected;
	
	m_pFrame->TransformationMatrix = m_pFrame->matExamples[iPose];
	m_pFrame->QuatRel = m_pFrame->quatExamples[iPose];

	UpdateFrames(m_pFrameRoot, NULL);
}



LPD3DXFRAME CExpMesh::FindParent(LPD3DXFRAME pFrame, LPCSTR BoneName)
{
	if (!pFrame) return NULL;

	LPD3DXFRAME pFrameChild = pFrame->pFrameFirstChild;
	while (pFrameChild)
	{
		if (pFrameChild->Name && !strcmp(pFrameChild->Name,BoneName) )
			return pFrame;
		pFrameChild = pFrameChild->pFrameSibling;
	}

	LPD3DXFRAME pFoundFrame;

	pFoundFrame = FindParent(pFrame->pFrameSibling, BoneName);
	if (!pFoundFrame) pFoundFrame = FindParent(pFrame->pFrameFirstChild, BoneName);

	return pFoundFrame;
}




HRESULT CExpMesh::UpdateDecl(IDirect3DDevice9* pd3dDevice)
{
	HRESULT hr;
	LPD3DXMESH pMesh = m_pMeshContainer->MeshData.pMesh;
	D3DVERTEXELEMENT9 pDecl[MAX_FVF_DECL_SIZE];
	D3DVERTEXELEMENT9 Decl[] = { 
		{0,  0, D3DDECLTYPE_FLOAT4,   0, D3DDECLUSAGE_POSITION, 1},
		{0,  0, D3DDECLTYPE_FLOAT3,   0, D3DDECLUSAGE_POSITION, 2},
		{0,  0, D3DDECLTYPE_FLOAT3,   0, D3DDECLUSAGE_POSITION, 3},
		{0,  0, D3DDECLTYPE_FLOAT3,   0, D3DDECLUSAGE_POSITION, 4},
		{0,  0, D3DDECLTYPE_FLOAT3,   0, D3DDECLUSAGE_NORMAL, 1},
		{0,  0, D3DDECLTYPE_FLOAT3,   0, D3DDECLUSAGE_NORMAL, 2},
		{0,  0, D3DDECLTYPE_FLOAT3,   0, D3DDECLUSAGE_NORMAL, 3},
		{0,  0, D3DDECLTYPE_FLOAT3,   0, D3DDECLUSAGE_NORMAL, 4},
	};
	INT i=0;
	DWORD dwStride = m_pMeshContainer->MeshData.pMesh->GetNumBytesPerVertex();

	V_RETURN(m_pMeshContainer->MeshData.pMesh->GetDeclaration(pDecl));

	pDecl[0].Type = D3DDECLTYPE_FLOAT3;

	while (pDecl[i].Stream != 0xff) ++i;

	pDecl[i+8] = pDecl[i];
	for (int j=0; j<8; ++j)
		pDecl[i+j]   = Decl[j];

	D3DXGenerateOutputDecl(pDecl,pDecl);
	V_RETURN(m_pMeshContainer->MeshData.pMesh->CloneMesh( 
		m_pMeshContainer->MeshData.pMesh->GetOptions(), 
		pDecl,
		pd3dDevice, &m_pMeshContainer->MeshData.pMesh  )
		);

	pMesh->Release();


	return S_OK;

}



HRESULT	CExpMesh::AfterLoad(IDirect3DDevice9* pd3dDevice)
{

	CQuatMesh::AfterLoad(pd3dDevice);


	if (m_pMeshContainer->pSkinInfo == NULL) return S_OK;


	// update deklaratoru, pro vice pozic a normal
	UpdateDecl(pd3dDevice);

	// Vyzere pozy, tzn. zkopiruje pozice kosti z examplu
	RetrieveExamples();

	
	OffsetVERTEX *pOffsets;
	D3DXVECTOR3 vMaxOffset, vMinOffset;

	// vygeneruje posunuti pozic a normal vrcholu ze poz
	GenerateExampleVertices(&pOffsets);

	// prirazeni pozic a normal do vertex bufferu modelu
	CompressPositionsAndNormals(pOffsets);

	delete[] pOffsets;

	// Vytvoreni interpolacnich objektu
	CreateInterpObjects((D3DXFRAME_EXP*)m_pFrameRoot);


	AdvanceTime(0.0f);

	return S_OK;

}


// Vypocet posunuti vrcholu (ExTranslation) v examplech
// Vypocitane podle rovnice (ExTransform)(vertex + ExTranslation) = ExVertex
//
// Predpoklad:
// Vsechny examply i a mesh, ktery obsahuje animaci museji mit stejne bone combination
//
//


void CExpMesh::GenerateExampleVertices(OffsetVERTEX **ppOffsets)
{

	D3DXMESHCONTAINER_QUAT* pMeshContainer = (D3DXMESHCONTAINER_QUAT*)m_pMeshContainer;
	Blend4TexUnLitExpVERTEX* pData;
	LPD3DXMESH pMesh = m_pMeshContainer->MeshData.pMesh;
	DWORD dwVertices = pMesh->GetNumVertices();
	BYTE* pIndices;
	FLOAT* pWeights;
	D3DXVECTOR3 vPos1, vPosEx;
	D3DXVECTOR3 vNorm1, vNormEx;
	VERTEX **ppVertices = new VERTEX*[m_listMesh.size()];
	DWORD dwStride = pMesh->GetNumBytesPerVertex();
	LPD3DXBONECOMBINATION pBoneComb;
	DWORD dwBoneCombID;

	*ppOffsets = new OffsetVERTEX[dwVertices];

	// zkopiruj vrcholy vsech examplu v example pozici
	for (int i=0; i<m_listMesh.size(); ++i)
		m_listMesh[i]->ProcessVertices(&ppVertices[i]);



	pBoneComb = reinterpret_cast<LPD3DXBONECOMBINATION>
						(m_pMeshContainer->pBoneCombinationBuf->GetBufferPointer());



	// projdi vsechny vrcholy a vypocitej posunuti v examplech
	if (pMesh->LockVertexBuffer(0,(void**)&pData) == D3D_OK)
	{
		for(DWORD i=0; i<dwVertices; ++i)
		{
			vPos1 = pData->position;
			vNorm1 = pData->normal; 
			pIndices = (BYTE*) &pData->indices;
			pWeights = (FLOAT*) &pData->blend1;

			dwBoneCombID = GetBoneCombID(i,pBoneComb);


			//
			// Zjisti index kosti, jejiz vahy jednotlivych poz se budou interpolovat
			// Vahy poz jsou pocitany podle potomku teto kosti
			//
			// Tato kost ma minimalni cislo v hierarchii ze vsech kosti, ktere ovlivnuji 
			// tento vrchol.
			//

			INT iBoneIndexForExamples = pIndices[0];

			for (int j=0; j<m_pMeshContainer->NumInfl; ++j)
				if ( m_pHierarchy[pIndices[j]] < m_pHierarchy[iBoneIndexForExamples] && pWeights[j] > 0.001)
					iBoneIndexForExamples = pIndices[j];

			(*ppOffsets)[i].iBone = iBoneIndexForExamples;

			//
			// Zjisti frame z indexu kosti
			//

			INT iBoneIndex = pBoneComb[dwBoneCombID].BoneId[iBoneIndexForExamples];
			LPD3DXFRAME pFrame = (D3DXFRAME_DERIVED*) GetFrameByName(m_pMeshContainer->pSkinInfo->GetBoneName(iBoneIndex));


			//
			// Tento frame sam o sobe informace a examplech nema (protoze je rizen examply svych potomku)
			// Jestli ma childa, najdeme info, ktere mu meni vrcholy v nem
			//

			D3DXFRAME_EXP* pFrameChild = (D3DXFRAME_EXP*) pFrame->pFrameFirstChild;

			// pro kazdy example, zjisti pozice kosti a untransformuj
			for ( int j=1; j<pFrameChild->iPoseCount; ++j)
			{

				INT iPoseIndex = pFrameChild->iPoseExampleIndex[j]-1;
				D3DXQUATERNION QuatFinal(0.0f,0.0f,0.0f,0.0f);
				D3DXQUATERNION Quats[4];
				D3DXQUATERNION QuatInverse;
				D3DXMATRIX MatFinal;
				INT iBoneIndex;
				D3DXFRAME_DERIVED* pFrame;

				// blending quaternionu
				for (int k=0; k<m_pMeshContainer->NumInfl; ++k)
				{
					iBoneIndex = pBoneComb[dwBoneCombID].BoneId[pIndices[k]];
					pFrame = (D3DXFRAME_DERIVED*) m_listMesh[iPoseIndex]->GetFrameByName(m_pMeshContainer->pSkinInfo->GetBoneName(iBoneIndex));
					assert(pFrame && "Nesedi nazvy kosti, muze byt rozdilna hierarchie");
					D3DXQuaternionMultiply(&Quats[k],&pMeshContainer->pBoneOffsetQuats[iBoneIndex],&pFrame->CombinedQuat);
					if (D3DXQuaternionDot(&Quats[k],&Quats[0]) < 0)
						Quats[k] *= -1;
				}

				for (int k=0; k<m_pMeshContainer->NumInfl; ++k)
				{
					QuatFinal += Quats[k]*pWeights[k];
				}

				D3DXQuaternionNormalize(&QuatFinal,&QuatFinal);
				D3DXQuaternionInverse(&QuatInverse,&QuatFinal);
				D3DXMatrixRotationQuaternion(&MatFinal,&QuatInverse);


				// Untransformuj examply do inicialni pozice a zjisti odchylku
				vPosEx = ppVertices[iPoseIndex][i].position;
				vNormEx = ppVertices[iPoseIndex][i].normal;

				// zjisti posunuti podle hlavni kosti
				D3DXVECTOR3 vTrans;
				iBoneIndex = pBoneComb[dwBoneCombID].BoneId[pIndices[0]];
				pFrame = (D3DXFRAME_DERIVED*) m_listMesh[iPoseIndex]->GetFrameByName(m_pMeshContainer->pSkinInfo->GetBoneName(iBoneIndex));
				vTrans.x = pFrame->CombinedTransformationMatrix._41;
				vTrans.y = pFrame->CombinedTransformationMatrix._42;
				vTrans.z = pFrame->CombinedTransformationMatrix._43;

				// transformace
				vPosEx -= vTrans;
				D3DXVec3TransformCoord(&vPosEx,&vPosEx,&MatFinal);
				D3DXVec3TransformNormal(&vNormEx,&vNormEx,&MatFinal);

				vPosEx -= vPos1;
				vNormEx -= vNorm1;

				(*ppOffsets)[i].PositionOffset[j-1] = vPosEx;
				(*ppOffsets)[i].NormalOffset[j-1] = vNormEx;

			}


			pData = (Blend4TexUnLitExpVERTEX*)(((BYTE*)pData) + dwStride);
		}

		pMesh->UnlockVertexBuffer();
	}

	// dealokuj pole zkopirovanych vrcholu
	for (int i=0; i<m_listMesh.size(); ++i)
		delete[] ppVertices[i];

	delete ppVertices;




}





void CExpMesh::CompressPositionsAndNormals(OffsetVERTEX *pOffset)
{
	D3DXMESHCONTAINER_QUAT* pMeshContainer = (D3DXMESHCONTAINER_QUAT*)m_pMeshContainer;
	Blend4TexUnLitExpVERTEX* pData;
	LPD3DXMESH pMesh = m_pMeshContainer->MeshData.pMesh;
	DWORD dwVertices = pMesh->GetNumVertices();
	DWORD dwStride = pMesh->GetNumBytesPerVertex();

	if (pMesh->LockVertexBuffer(0,(void**)&pData) == D3D_OK)
	{
		for(DWORD i=0; i<dwVertices; ++i)
		{

			pData->position1tr = D3DXVECTOR4(pOffset[i].PositionOffset[0].x, pOffset[i].PositionOffset[0].y, pOffset[i].PositionOffset[0].z,
											pOffset[i].iBone);
			pData->norm1ofs = pOffset[i].NormalOffset[0];


			D3DXVECTOR3 *pCompressedPos = &pData->position2tr;
			D3DXVECTOR3 *pCompressedNorm = &pData->norm2ofs;

			for( int j=1; j<MAX_EXAMPLES; ++j)
			{

				// komprese
				*pCompressedPos = pOffset[i].PositionOffset[j];
				*pCompressedNorm = pOffset[i].NormalOffset[j];


				++pCompressedPos;
				++pCompressedNorm;
			}
			pData = (Blend4TexUnLitExpVERTEX*)(((BYTE*)pData) + dwStride);
		}
		pMesh->UnlockVertexBuffer();
	}
}




void CExpMesh::RetrieveExamples()
{
	LPD3DXFRAME *ppRootFrames = new LPD3DXFRAME[m_listMesh.size()];
	bool bCopy;

	// zjisti nastaveni hierarchie vsech examplu, 0 je zakladni pozice
	for (int i=0; i<m_listMesh.size(); ++i)
	{
		bCopy = IsExamplePose((D3DXFRAME_EXP*)m_pFrameRoot, m_listMesh[i]->GetRootFrame());
		RetrieveExampleInFrame(i+1,bCopy,(D3DXFRAME_EXP*)m_pFrameRoot,(D3DXFRAME_QUAT*)m_listMesh[i]->GetRootFrame());
	}
}



void CExpMesh::RetrieveExampleInFrame(INT iExampleNum, bool bCopy, D3DXFRAME_EXP* pFrame,D3DXFRAME_QUAT* pFrameEx)
{

	if (bCopy)
	{
		pFrame->iPoseExampleIndex[pFrame->iPoseCount] = iExampleNum;
		pFrame->vecExamples[pFrame->iPoseCount] = 
			D3DXVECTOR3(pFrameEx->TransformationMatrix._11,pFrameEx->TransformationMatrix._12, pFrameEx->TransformationMatrix._13);
		pFrame->quatExamples[pFrame->iPoseCount] = pFrameEx->QuatRel;
		pFrame->matExamples[pFrame->iPoseCount] = pFrameEx->TransformationMatrix;


		pFrame->iPoseExampleIndex[0] = 0;
		pFrame->vecExamples[0] = 
			D3DXVECTOR3(pFrame->TransformationMatrix._11, pFrame->TransformationMatrix._12, pFrame->TransformationMatrix._13);
		pFrame->quatExamples[0] = pFrame->QuatRel;
		pFrame->matExamples[0] = pFrame->TransformationMatrix;


		++pFrame->iPoseCount;
	}

	if (pFrame->pFrameSibling != NULL)
	{
		RetrieveExampleInFrame(iExampleNum, bCopy, (D3DXFRAME_EXP*)pFrame->pFrameSibling, (D3DXFRAME_QUAT*)pFrameEx->pFrameSibling);
	}

	if (pFrame->pFrameFirstChild != NULL)
	{
		bCopy = IsExamplePose((D3DXFRAME_EXP*)pFrame->pFrameFirstChild, (D3DXFRAME_QUAT*)pFrameEx->pFrameFirstChild);
		RetrieveExampleInFrame(iExampleNum, bCopy, (D3DXFRAME_EXP*)pFrame->pFrameFirstChild, (D3DXFRAME_QUAT*)pFrameEx->pFrameFirstChild);
	}

}



bool CExpMesh::IsExamplePose(D3DXFRAME_EXP* pFrame,LPD3DXFRAME pFrameExBase)
{
	D3DXFRAME_DERIVED* pFrameEx = (D3DXFRAME_DERIVED*) pFrameExBase;

	while (pFrame)
	{
		if ( !QuaternionEquals(&pFrame->QuatRel,&pFrameEx->QuatRel) ) return true;
		pFrame = (D3DXFRAME_EXP*) pFrame->pFrameSibling;
		pFrameEx = (D3DXFRAME_DERIVED*) pFrameEx->pFrameSibling;
	}

	return false;
}



void CExpMesh::CreateInterpObjects(D3DXFRAME_EXP *pFrame)
{
	if (pFrame == NULL) return;

	// jestlize je to pojmenovana kost, vytvor interpolacni tridu
	if (pFrame->Name && strcmp(pFrame->Name,"") ) 
	{
	
		FLOAT* pSigmas = new FLOAT[pFrame->iPoseCount];

		for(int i=0; i<pFrame->iPoseCount; ++i)
				pSigmas[i] = m_pConfigFile->GetData(pFrame->Name,i);

		pFrame->pInterp = new CRBFInterp(pFrame->iPoseCount,pFrame->vecExamples,pSigmas);

		delete[] pSigmas;

	}

	CreateInterpObjects((D3DXFRAME_EXP*) pFrame->pFrameSibling);
	CreateInterpObjects((D3DXFRAME_EXP*) pFrame->pFrameFirstChild);
}




void CExpMesh::SetBonesToEffect(IDirect3DDevice9 *pd3dDevice, UINT iAttrib, LPD3DXMESHCONTAINER pMeshContainerBase)
{
	//	CMesh::SetBonesToEffect(iAttrib,pMeshContainerBase);
	D3DXMESHCONTAINER_EXP *pMeshContainer = (D3DXMESHCONTAINER_EXP*)pMeshContainerBase;
	HRESULT hr;
	UINT iMatrixIndex;
	UINT iPaletteEntry;
	LPD3DXBONECOMBINATION pBoneComb;

	pBoneComb = reinterpret_cast<LPD3DXBONECOMBINATION>(pMeshContainer->pBoneCombinationBuf->GetBufferPointer());

	// first calculate all the world matrices
	for (iPaletteEntry = 0; iPaletteEntry < pMeshContainer->NumPaletteEntries; ++iPaletteEntry)
	{
		iMatrixIndex = pBoneComb[iAttrib].BoneId[iPaletteEntry];
		if (iMatrixIndex != UINT_MAX)
		{

			m_pBoneQuats[iPaletteEntry] = *pMeshContainer->ppBoneQuatPtrs[iMatrixIndex];
//			D3DXQuaternionMultiply(&m_pBoneQuats[iPaletteEntry],&pMeshContainer->pBoneOffsetQuats[iMatrixIndex], pMeshContainer->ppBoneQuatPtrs[iMatrixIndex]);						
			m_pTranslations[iPaletteEntry] = *pMeshContainer->ppBoneTransPtrs[iMatrixIndex];


			m_pExampleWeights[iPaletteEntry] = *pMeshContainer->ppExampleWeightsPtrs[iMatrixIndex];

			if (iMatrixIndex == m_iBoneExIndex)
				V( m_pSM->GetD3DXEffect()->SetInt( "iBoneDebug", iPaletteEntry ) );
		}


 	}
	if (-1 == m_iBoneExIndex)
		V( m_pSM->GetD3DXEffect()->SetInt( "iBoneDebug", -1 ) );

	V( m_pSM->GetD3DXEffect()->SetVectorArray( "WorldQuatArray", (D3DXVECTOR4*) m_pBoneQuats, pMeshContainer->NumPaletteEntries) );
	V( m_pSM->GetD3DXEffect()->SetVectorArray( "WorldTrans", m_pTranslations, pMeshContainer->NumPaletteEntries) );
	V( m_pSM->GetD3DXEffect()->SetVectorArray( "ExampleWeights", m_pExampleWeights, pMeshContainer->NumPaletteEntries) );
	V( m_pSM->GetD3DXEffect()->SetInt( "ShowExVert", m_bShowExVert == true ? 1 : 0) );



}
