/*! \file 
    \brief Tda modelu pro metodu SJBBE
*/ 

#pragma once

#include "CQuatMesh.h"
#include "CVirtualInterp.h"
#include <vector>



class CConfigFile;

/// Maximln poet vzorovch pz na kost
const int MAX_EXAMPLES = 4;

typedef std::vector<CVirtualMesh*> MeshList;
typedef std::vector<CVirtualMesh*>::iterator MeshListIt;

/// Struktura kosti pro tdu CExpMesh
struct D3DXFRAME_EXP: public D3DXFRAME_QUAT
{
    /// Vzorov pzy - osy kosti v tchto pzch
	D3DXVECTOR3		vecExamples[MAX_EXAMPLES+1];
    /// Vzorov pzy - rotace kosti v tchto pzch    
	D3DXQUATERNION  quatExamples[MAX_EXAMPLES+1];
    /// Vzorov pzy - matice kosti v tchto pzch        
	D3DXMATRIX		matExamples[MAX_EXAMPLES+1];
    /// Index vzorov pzy do seznamu vech vzorovch pz
	INT				iPoseExampleIndex[MAX_EXAMPLES+1];
    /// Poet vzorovch pz
	INT				iPoseCount;
    /// Vzdlenost od vzorovch pz
	FLOAT			fRelLength[MAX_EXAMPLES+1];
    /// Vhy jednotlivch vzorovch pz
	D3DXVECTOR4		fExampleWeights;
    /// Odkaz na interpolan tdu
	CVirtualInterp	*pInterp;

	D3DXFRAME_EXP() 
	{
		// vzdy je alespon zakladni poza
		iPoseCount = 1;
		pInterp = NULL;

		for (int i=0; i<MAX_EXAMPLES+1; ++i)
		{
			iPoseExampleIndex[i] = -1;
			vecExamples[i] = D3DXVECTOR3(0.0f,0.0f,0.0f);
			fRelLength[i] = 0.0f;
		}

	}

	~D3DXFRAME_EXP() 
	{
		delete pInterp;
	}
};

/// Mesh kontejner pro tdu CExpMesh
struct D3DXMESHCONTAINER_EXP : public D3DXMESHCONTAINER_QUAT
{
    /// Pole vech odkaz na vhy jednotlivch pz
	D3DXVECTOR4** ppExampleWeightsPtrs;

	D3DXMESHCONTAINER_EXP()
	{
		ppExampleWeightsPtrs = NULL;
	}

	~D3DXMESHCONTAINER_EXP()
	{
		SAFE_DELETE_ARRAY( ppExampleWeightsPtrs );
	}

};



struct OffsetVERTEX
{
	INT			iBone;
	D3DXVECTOR3 PositionOffset[MAX_EXAMPLES];
	D3DXVECTOR3 NormalOffset[MAX_EXAMPLES];

	OffsetVERTEX()
	{
		for (int i=0; i<MAX_EXAMPLES; ++i)
		{
			PositionOffset[i] = D3DXVECTOR3(0.0f,0.0f,0.0f);
			NormalOffset[i] = D3DXVECTOR3(0.0f,0.0f,0.0f);
		}
	}
};



/// Tda modelu pro metodu SJBBE
class CExpMesh : 
		public CQuatMesh

{
public:

	CExpMesh(std::wstring filename, bool weld, bool optimize=true);
	virtual ~CExpMesh();

	virtual HRESULT	OnCreateDevice(IDirect3DDevice9* pd3dDevice, DWORD dwBehaviorFlags);
	virtual void	OnDestroyDevice();

	virtual HRESULT	AfterLoad(IDirect3DDevice9* pd3dDevice);

	virtual D3DXFRAME_BASE*			GetNewFrame() { return new D3DXFRAME_EXP; }
	virtual D3DXMESHCONTAINER_BASE*	GetNewMeshContainer() { return new D3DXMESHCONTAINER_EXP; }

	virtual void SelectFrame(LPCSTR BoneName);

	void SetPoseToSelectedFrame(int iPose);
	void ShowExampleVertices(bool bShow) {m_bShowExVert = bShow;}



protected:

	void	LoadExamples();

	void	AllocForBones(int NumBones);
	void	FreeForBones();
	void	SetupBonePointersOnMeshAlloc(LPD3DXMESHCONTAINER pMeshContainerBase, int iBones);
	void	SetupBonePointersOnMeshCB(LPD3DXMESHCONTAINER pMeshContainerBase, LPD3DXFRAME pFrameBase, int iBone);

	virtual void	UpdateFramesPreCB(LPD3DXFRAME pFrameBase,LPD3DXFRAME pFrameParent);
	virtual void	UpdateFramesPostCB(LPD3DXFRAME pFrameBase,LPD3DXFRAME pFrameParent);
	virtual void	SetBonesToEffect(IDirect3DDevice9 *pd3dDevice, UINT iAttrib, LPD3DXMESHCONTAINER pMeshContainerBase);

	HRESULT			UpdateDecl(IDirect3DDevice9* pd3dDevice);
	void			GenerateExampleVertices(OffsetVERTEX **ppOffsets);
	void			RetrieveExamples();
	void			RetrieveExampleInFrame(INT iExampleNum, bool bCopy, D3DXFRAME_EXP* pFrame,D3DXFRAME_QUAT* pFrameEx);
	bool			IsExamplePose(D3DXFRAME_EXP* pFrame,LPD3DXFRAME pFrameEx);

	void			CreateInterpObjects(D3DXFRAME_EXP *pFrame);

	void			CompressPositionsAndNormals(OffsetVERTEX *pOffset);

	LPD3DXFRAME		FindParent(LPD3DXFRAME pFrame, LPCSTR BoneName);


	INT				m_iBoneExIndex;
	MeshList		m_listMesh;
	D3DXVECTOR4		*m_pExampleWeights;
	CConfigFile		*m_pConfigFile;
	bool			m_bShowExVert;




};






