#include "dxstdafx.h"


#include "CFixedIndexedMesh.h"
#include "CAllocateHierarchyFixedIndexed.h"


void CFixedIndexedMesh::SetBonesToEffect( IDirect3DDevice9 *pd3dDevice, UINT iAttrib, LPD3DXMESHCONTAINER pMeshContainerBase)
{
}


ID3DXAllocateHierarchy* CFixedIndexedMesh::GetNewAlloc()
{
	return new CAllocateHierarchyFixedIndexed(this);
}



void CFixedIndexedMesh::DrawMeshContainer( IDirect3DDevice9 *pd3dDevice, LPD3DXMESHCONTAINER pMeshContainerBase, LPD3DXFRAME pFrameBase)
{
    D3DXMESHCONTAINER_DERIVED *pMeshContainer = (D3DXMESHCONTAINER_DERIVED*)pMeshContainerBase;
    D3DXFRAME_DERIVED *pFrame = (D3DXFRAME_DERIVED*)pFrameBase;
	HRESULT hr;
    UINT iAttribIdPrev;
	UINT iMatrixIndex;
    LPD3DXBONECOMBINATION pBoneComb;
	D3DXMATRIX matTemp;

	if (pMeshContainer->pSkinInfo == NULL) 
	{

		// render bez vertex shaderu
		V( pd3dDevice->SetTransform(D3DTS_WORLD, &pFrame->CombinedTransformationMatrix) );

		for (int iMaterial = 0; iMaterial < pMeshContainer->NumMaterials; iMaterial++)
		{
			V( pd3dDevice->SetMaterial( &pMeshContainer->pMaterials[iMaterial].MatD3D ) );
			V( pd3dDevice->SetTexture( 0, pMeshContainer->ppTextures[iMaterial] ) );
			V( pMeshContainer->MeshData.pMesh->DrawSubset(iMaterial) );
		}
		return;
	}


	iAttribIdPrev = UNUSED32; 
	pBoneComb = reinterpret_cast<LPD3DXBONECOMBINATION>(pMeshContainer->pBoneCombinationBuf->GetBufferPointer());

	// if hw doesn't support indexed vertex processing, switch to software vertex processing
	if (pMeshContainer->UseSoftwareVP)
	{
		// If hw or pure hw vertex processing is forced, we can't render the
		// mesh, so just exit out.  Typical applications should create
		// a device with appropriate vertex processing capability for this
		// skinning method.
		if( m_dwBehaviorFlags & D3DCREATE_HARDWARE_VERTEXPROCESSING )
		{
			m_ErrMessage = L"Hardware acceleration not supported";
			return;
		}

		V( pd3dDevice->SetSoftwareVertexProcessing(TRUE) );
	}

	// set the number of vertex blend indices to be blended
	if (pMeshContainer->NumInfl == 1)
	{
		V( pd3dDevice->SetRenderState(D3DRS_VERTEXBLEND, D3DVBF_0WEIGHTS) );
	}
	else
	{
		V( pd3dDevice->SetRenderState(D3DRS_VERTEXBLEND, pMeshContainer->NumInfl - 1) );
	}

	if (pMeshContainer->NumInfl)
		V( pd3dDevice->SetRenderState(D3DRS_INDEXEDVERTEXBLENDENABLE, TRUE) );

	// for each attribute group in the mesh, calculate the set of matrices in the palette and then draw the mesh subset
	pBoneComb = reinterpret_cast<LPD3DXBONECOMBINATION>(pMeshContainer->pBoneCombinationBuf->GetBufferPointer());
	for (UINT iAttrib = 0; iAttrib < pMeshContainer->NumAttributeGroups; iAttrib++)
	{
		// first calculate all the world matrices
		for (UINT iPaletteEntry = 0; iPaletteEntry < pMeshContainer->NumPaletteEntries; ++iPaletteEntry)
		{
			iMatrixIndex = pBoneComb[iAttrib].BoneId[iPaletteEntry];
			if (iMatrixIndex != UINT_MAX)
			{
				D3DXMatrixMultiply( &matTemp, &pMeshContainer->pBoneOffsetMatrices[iMatrixIndex], pMeshContainer->ppBoneMatrixPtrs[iMatrixIndex] );
				V( pd3dDevice->SetTransform( D3DTS_WORLDMATRIX( iPaletteEntry ), &matTemp ) );
			}
		}

		// setup the material of the mesh subset - REMEMBER to use the original pre-skinning attribute id to get the correct material id
		V( pd3dDevice->SetMaterial( &pMeshContainer->pMaterials[pBoneComb[iAttrib].AttribId].MatD3D ) );
		V( pd3dDevice->SetTexture( 0, pMeshContainer->ppTextures[pBoneComb[iAttrib].AttribId] ) );

		// finally draw the subset with the current world matrix palette and material state
		V( pMeshContainer->MeshData.pMesh->DrawSubset( iAttrib ) );
	}

	// reset blending state
	V( pd3dDevice->SetRenderState(D3DRS_INDEXEDVERTEXBLENDENABLE, FALSE) );
	V( pd3dDevice->SetRenderState(D3DRS_VERTEXBLEND, 0) );

	// remember to reset back to hw vertex processing if software was required
	if (pMeshContainer->UseSoftwareVP)
	{
		V( pd3dDevice->SetSoftwareVertexProcessing(FALSE) );
	}

}




