#include "dxstdafx.h"


#include "CFixedMesh.h"
#include "CAllocateHierarchyFixed.h"


void CFixedMesh::SetBonesToEffect( IDirect3DDevice9 *pd3dDevice, UINT iAttrib, LPD3DXMESHCONTAINER pMeshContainerBase)
{
}



ID3DXAllocateHierarchy* CFixedMesh::GetNewAlloc()
{
	return new CAllocateHierarchyFixed(this);
}


void CFixedMesh::DrawMeshContainer( IDirect3DDevice9 *pd3dDevice, LPD3DXMESHCONTAINER pMeshContainerBase, LPD3DXFRAME pFrameBase)
{
    D3DXMESHCONTAINER_DERIVED *pMeshContainer = (D3DXMESHCONTAINER_DERIVED*)pMeshContainerBase;
    D3DXFRAME_DERIVED *pFrame = (D3DXFRAME_DERIVED*)pFrameBase;
	HRESULT hr;
    UINT iAttribIdPrev;
	UINT iMatrixIndex;
    LPD3DXBONECOMBINATION pBoneComb;
	D3DXMATRIX matTemp;


	if (pMeshContainer->pSkinInfo == NULL) 
	{

		// render bez vertex shaderu
		V( pd3dDevice->SetTransform(D3DTS_WORLD, &pFrame->CombinedTransformationMatrix) );

		for (int iMaterial = 0; iMaterial < pMeshContainer->NumMaterials; iMaterial++)
		{
			V( pd3dDevice->SetMaterial( &pMeshContainer->pMaterials[iMaterial].MatD3D ) );
			V( pd3dDevice->SetTexture( 0, pMeshContainer->ppTextures[iMaterial] ) );
			V( pMeshContainer->MeshData.pMesh->DrawSubset(iMaterial) );
		}
		return;
	}


	INT NumBlend;
	iAttribIdPrev = UNUSED32; 
	pBoneComb = reinterpret_cast<LPD3DXBONECOMBINATION>(pMeshContainer->pBoneCombinationBuf->GetBufferPointer());


	// if hw doesn't support indexed vertex processing, switch to software vertex processing
	if (pMeshContainer->UseSoftwareVP)
	{
		// If hw or pure hw vertex processing is forced, we can't render the
		// mesh, so just exit out.  Typical applications should create
		// a device with appropriate vertex processing capability for this
		// skinning method.
		if( m_dwBehaviorFlags & D3DCREATE_HARDWARE_VERTEXPROCESSING )
		{
			m_ErrMessage = L"Hardware acceleration not supported";
			return;
		}

		V( pd3dDevice->SetSoftwareVertexProcessing(TRUE) );
	}

	// Draw using default vtx processing of the device (typically HW)
	for (UINT iAttrib = 0; iAttrib < pMeshContainer->NumAttributeGroups; iAttrib++)
	{
		NumBlend = 0;
		for (DWORD i = 0; i < pMeshContainer->NumInfl; ++i)
		{
			if (pBoneComb[iAttrib].BoneId[i] != UINT_MAX)
			{
				NumBlend = i;
			}
		}

		// first calculate the world matrices for the current set of blend weights and get the accurate count of the number of blends
		for (DWORD i = 0; i < pMeshContainer->NumInfl; ++i)
		{
			iMatrixIndex = pBoneComb[iAttrib].BoneId[i];
			if (iMatrixIndex != UINT_MAX)
			{
				D3DXMatrixMultiply( &matTemp, &pMeshContainer->pBoneOffsetMatrices[iMatrixIndex], pMeshContainer->ppBoneMatrixPtrs[iMatrixIndex] );
				V( pd3dDevice->SetTransform( D3DTS_WORLDMATRIX( i ), &matTemp ) );
			}
		}

		V( pd3dDevice->SetRenderState(D3DRS_VERTEXBLEND, NumBlend) );

		// lookup the material used for this subset of faces
		if ((iAttribIdPrev != pBoneComb[iAttrib].AttribId) || (iAttribIdPrev == UNUSED32))
		{
			V( pd3dDevice->SetMaterial( &pMeshContainer->pMaterials[pBoneComb[iAttrib].AttribId].MatD3D ) );
			V( pd3dDevice->SetTexture( 0, pMeshContainer->ppTextures[pBoneComb[iAttrib].AttribId] ) );
			iAttribIdPrev = pBoneComb[iAttrib].AttribId;
		}

		// draw the subset now that the correct material and matrices are loaded
		V( pMeshContainer->MeshData.pMesh->DrawSubset(iAttrib) );
	}

	V( pd3dDevice->SetRenderState( D3DRS_VERTEXBLEND, 0) );

	// remember to reset back to hw vertex processing if software was required
	if (pMeshContainer->UseSoftwareVP)
	{
		V( pd3dDevice->SetSoftwareVertexProcessing(FALSE) );
	}

}


/*

void CFixedMesh::DrawMeshContainer( IDirect3DDevice9 *pd3dDevice, LPD3DXMESHCONTAINER pMeshContainerBase, LPD3DXFRAME pFrameBase)
{
	D3DXMESHCONTAINER_DERIVED *pMeshContainer = (D3DXMESHCONTAINER_DERIVED*)pMeshContainerBase;
	D3DXFRAME_DERIVED *pFrame = (D3DXFRAME_DERIVED*)pFrameBase;
	HRESULT hr;
	UINT iAttribIdPrev;
	UINT iMatrixIndex;
	LPD3DXBONECOMBINATION pBoneComb;
	D3DXMATRIX matTemp;

	D3DCOLORVALUE barvy[20];
	barvy[0].r = 1.0f;
	barvy[0].g = 0.0f;
	barvy[0].b = 0.0f;

	barvy[1].r = 0.0f;
	barvy[1].g = 1.0f;
	barvy[1].b = 0.0f;

	barvy[2].r = 0.0f;
	barvy[2].g = 0.0f;
	barvy[2].b = 1.0f;

	barvy[3].r = 1.0f;
	barvy[3].g = 1.0f;
	barvy[3].b = 0.0f;

	barvy[4].r = 1.0f;
	barvy[4].g = 0.0f;
	barvy[4].b = 1.0f;

	barvy[5].r = 0.0f;
	barvy[5].g = 1.0f;
	barvy[5].b = 1.0f;

	barvy[6].r = 1.0f;
	barvy[6].g = 1.0f;
	barvy[6].b = 1.0f;

	barvy[7].r = 1.0f;
	barvy[7].g = 0.0f;
	barvy[7].b = 0.0f;

	barvy[8].r = 0.0f;
	barvy[8].g = 1.0f;
	barvy[8].b = 0.0f;

	barvy[9].r = 0.0f;
	barvy[9].g = 0.0f;
	barvy[9].b = 1.0f;

	barvy[10].r = 1.0f;
	barvy[10].g = 1.0f;
	barvy[10].b = 0.0f;

	barvy[11].r = 1.0f;
	barvy[11].g = 0.0f;
	barvy[11].b = 1.0f;

	barvy[12].r = 0.0f;
	barvy[12].g = 1.0f;
	barvy[12].b = 1.0f;

	barvy[13].r = 1.0f;
	barvy[13].g = 1.0f;
	barvy[13].b = 1.0f;

	barvy[14].r = 1.0f;
	barvy[14].g = 0.0f;
	barvy[14].b = 0.0f;

	barvy[15].r = 0.0f;
	barvy[15].g = 1.0f;
	barvy[15].b = 0.0f;

	barvy[16].r = 0.0f;
	barvy[16].g = 0.0f;
	barvy[16].b = 1.0f;

	barvy[17].r = 1.0f;
	barvy[17].g = 1.0f;
	barvy[17].b = 0.0f;

	barvy[18].r = 1.0f;
	barvy[18].g = 0.0f;
	barvy[18].b = 1.0f;

	barvy[19].r = 0.0f;
	barvy[19].g = 1.0f;
	barvy[19].b = 1.0f;



	if (pMeshContainer->pSkinInfo == NULL) 
	{

		// render bez vertex shaderu
		V( pd3dDevice->SetTransform(D3DTS_WORLD, &pFrame->CombinedTransformationMatrix) );

		for (int iMaterial = 0; iMaterial < pMeshContainer->NumMaterials; iMaterial++)
		{
			V( pd3dDevice->SetMaterial( &pMeshContainer->pMaterials[iMaterial].MatD3D ) );
			V( pd3dDevice->SetTexture( 0, pMeshContainer->ppTextures[iMaterial] ) );
			V( pMeshContainer->MeshData.pMesh->DrawSubset(iMaterial) );
		}
		return;
	}


	INT NumBlend;
	iAttribIdPrev = UNUSED32; 
	pBoneComb = reinterpret_cast<LPD3DXBONECOMBINATION>(pMeshContainer->pBoneCombinationBuf->GetBufferPointer());


	// if hw doesn't support indexed vertex processing, switch to software vertex processing
	if (pMeshContainer->UseSoftwareVP)
	{
		// If hw or pure hw vertex processing is forced, we can't render the
		// mesh, so just exit out.  Typical applications should create
		// a device with appropriate vertex processing capability for this
		// skinning method.
		if( m_dwBehaviorFlags & D3DCREATE_HARDWARE_VERTEXPROCESSING )
		{
			m_ErrMessage = L"Hardware acceleration not supported";
			return;
		}

		V( pd3dDevice->SetSoftwareVertexProcessing(TRUE) );
	}

	// Draw using default vtx processing of the device (typically HW)
	for (UINT iAttrib = 0; iAttrib < pMeshContainer->NumAttributeGroups; iAttrib++)
	{
		NumBlend = 0;
		for (DWORD i = 0; i < pMeshContainer->NumInfl; ++i)
		{
			if (pBoneComb[iAttrib].BoneId[i] != UINT_MAX)
			{
				NumBlend = i;
			}
		}

		// first calculate the world matrices for the current set of blend weights and get the accurate count of the number of blends
		for (DWORD i = 0; i < pMeshContainer->NumInfl; ++i)
		{
			iMatrixIndex = pBoneComb[iAttrib].BoneId[i];
			if (iMatrixIndex != UINT_MAX)
			{
				D3DXMatrixMultiply( &matTemp, &pMeshContainer->pBoneOffsetMatrices[iMatrixIndex], pMeshContainer->ppBoneMatrixPtrs[iMatrixIndex] );
				V( pd3dDevice->SetTransform( D3DTS_WORLDMATRIX( i ), &matTemp ) );
			}
		}

		V( pd3dDevice->SetRenderState(D3DRS_VERTEXBLEND, NumBlend) );

		// lookup the material used for this subset of faces
		//	if ((iAttribIdPrev != pBoneComb[iAttrib].AttribId) || (iAttribIdPrev == UNUSED32))
		{
			// Sum of all ambient and emissive contribution
			D3DXCOLOR color1(pMeshContainer->pMaterials[pBoneComb[iAttrib].AttribId].MatD3D.Ambient);
			D3DXCOLOR color2(.25, .25, .25, 1.0);
			D3DXCOLOR ambEmm;

			D3DXColorModulate(&ambEmm, &color1, &color2);
			ambEmm += D3DXCOLOR(pMeshContainer->pMaterials[pBoneComb[iAttrib].AttribId].MatD3D.Emissive);

			pMeshContainer->pMaterials[pBoneComb[iAttrib].AttribId].MatD3D.Diffuse.r = barvy[iAttrib].r;
			pMeshContainer->pMaterials[pBoneComb[iAttrib].AttribId].MatD3D.Diffuse.g = barvy[iAttrib].g;
			pMeshContainer->pMaterials[pBoneComb[iAttrib].AttribId].MatD3D.Diffuse.b = barvy[iAttrib].b;

			V( pd3dDevice->SetMaterial( &pMeshContainer->pMaterials[pBoneComb[iAttrib].AttribId].MatD3D ) );
			V( pd3dDevice->SetTexture( 0, pMeshContainer->ppTextures[pBoneComb[iAttrib].AttribId] ) );
			iAttribIdPrev = pBoneComb[iAttrib].AttribId;
		}

		// draw the subset now that the correct material and matrices are loaded
		V( pMeshContainer->MeshData.pMesh->DrawSubset(iAttrib) );
	}

	V( pd3dDevice->SetRenderState( D3DRS_VERTEXBLEND, 0) );

	// remember to reset back to hw vertex processing if software was required
	if (pMeshContainer->UseSoftwareVP)
	{
		V( pd3dDevice->SetSoftwareVertexProcessing(FALSE) );
	}

}

*/