/*! \file 
    \brief Model pro vertex bleding metody VSP1, VSP2, VSP3
*/ 

#pragma once

#include "CVirtualMesh.h"
#include <string>
#include <vector>

/// Struktura kosti pro tdu CMesh
struct D3DXFRAME_DERIVED: public D3DXFRAME_BASE
{
    /// Transformace kosti v referenn pze
	D3DXMATRIX			InitialTransformationMatrix;
    /// Transformace kosti vzhledem k souadn soustav modelu
	D3DXMATRIX			CombinedTransformationMatrix;
    /// Relativn rotace vzhledem k pedchdci
	D3DXQUATERNION		QuatRel;
    /// Relativn translace vzhledem k pedchdci
	D3DXVECTOR3			TransRel;
    /// Relativn mtko vzhledem k pedchdci
	D3DXVECTOR3			ScaleRel;
    /// Rotace kosti vzhledem k souadn soustav modelu
	D3DXQUATERNION		CombinedQuat;
    /// Translace kosti vzhledem k souadn soustav modelu
	D3DXVECTOR4			CombinedTrans;
};


/// Mesh kontejner pro tdu CMesh
struct D3DXMESHCONTAINER_DERIVED: public D3DXMESHCONTAINER_BASE
{
  /// Pole vech odkaz na matice v kostech
	D3DXMATRIX**         ppBoneMatrixPtrs;

	D3DXMESHCONTAINER_DERIVED()
	{
		ppBoneMatrixPtrs = NULL;
	}

	~D3DXMESHCONTAINER_DERIVED()
	{
		SAFE_DELETE_ARRAY( ppBoneMatrixPtrs );
	}
};


/// Tda modelu pro vertex bleding metody VSP1, VSP2, VSP3.
class CMesh : 
		public CVirtualMesh,
		public CAllocationMeshCB
{
public:

	CMesh(std::wstring filename, bool Weld, bool Optimize = true);
	virtual ~CMesh();

	// CVirtualMesh

	HRESULT	OnCreateDevice(IDirect3DDevice9* pd3dDevice, DWORD dwBehaviorFlags);
	void	OnDestroyDevice();
	LPD3DXFRAME				GetRootFrame() { return m_pFrameRoot; }
	D3DXMESHCONTAINER_BASE*	GetMeshContainer() { return m_pMeshContainer; }
	LPD3DXFRAME				GetFrameByName(LPCSTR BoneName);
	void		SetSkinningMethod(CSkinningMethod* pSM) {m_pSM = pSM;}
	void		Render(IDirect3DDevice9* pd3dDevice);
	void		AdvanceTime(double fElapsedTime);
	void  		SetTime(double fElapsedTime) { AdvanceTime(fElapsedTime); }
	void		ProcessVertices(VERTEX **ppVertices);
	float		GetObjectRadius() { return m_fObjectRadius; }
	D3DXVECTOR3	GetObjectCenter() { return m_vObjectCenter; }
	double		GetAnimationTime() { 
					if (m_pAnimController) return m_pAnimController->GetTime(); 
					else return 0.0;
				}
	void	GetFrameNames(std::vector<std::wstring> &FrameNames);
	void	SelectFrame(LPCSTR BoneName);
	void	TransformSelectedFrame(D3DXMATRIX *mat);
	void	SetInitialPose();
	INT		GetNumVertices()  { return m_pMeshContainer->MeshData.pMesh->GetNumVertices(); }
	INT		GetNumFaces()		{ return m_pMeshContainer->MeshData.pMesh->GetNumFaces(); }
	INT		GetNumBonesPerVertex() { return m_pMeshContainer->NumInfl; }
	CMeshMenu* GetMeshMenu() { return m_pMeshMenu; }
	std::wstring GetErrMessage()  { return m_ErrMessage; }

	// !CVirtualMesh

	// CAllocationMeshCB

	virtual void	AllocForBones(int m_NumBoneMatricesMax);
	virtual void	FreeForBones();
	virtual D3DXFRAME_BASE*			GetNewFrame() { return new D3DXFRAME_DERIVED; }
	virtual D3DXMESHCONTAINER_BASE*	GetNewMeshContainer() { return new D3DXMESHCONTAINER_DERIVED; }
	// !CAllocationMeshCB



protected:

	virtual HRESULT	AfterLoad(IDirect3DDevice9* pd3dDevice) { return S_OK;}
	virtual HRESULT	Optimize(IDirect3DDevice9 *pd3dDevice);

	// vyhledani a nastaveni kosti
	HRESULT			SetupBonePointers( LPD3DXFRAME pFrame );
	HRESULT			SetupBonePointersOnMesh( LPD3DXMESHCONTAINER pMeshContainerBase, LPD3DXFRAME pFrame );
	virtual void	SetupBonePointersOnMeshAlloc(LPD3DXMESHCONTAINER pMeshContainerBase, int iBones);
	virtual void	SetupBonePointersOnMeshCB(LPD3DXMESHCONTAINER pMeshContainerBase, LPD3DXFRAME pFrameBase, int iBone);


	// update kosti po zmene casu
	void			UpdateFrames( LPD3DXFRAME pFrame, LPD3DXFRAME pFrameParent);
	virtual void	UpdateFramesPreCB(LPD3DXFRAME pFrameBase,LPD3DXFRAME pFrameParent);
	virtual void	UpdateFramesPostCB(LPD3DXFRAME pFrameBase,LPD3DXFRAME pFrameParent) {}

	// nastaveni promennych v efekt souboru
	virtual void	SetBonesToEffect(IDirect3DDevice9 *pd3dDevice, UINT iAttrib, LPD3DXMESHCONTAINER pMeshContainerBase);

	// transformace vrcholu
	virtual void	ProcessVerticesSkinned(VERTEX **ppVertices);
	void			ProcessVerticesRigid(VERTEX **ppVertices);

	void			DrawFrame(IDirect3DDevice9* pd3dDevice, D3DXFRAME *pFrame);
	virtual void	DrawMeshContainer(IDirect3DDevice9 *pd3dDevice, LPD3DXMESHCONTAINER pMeshContainerBase, LPD3DXFRAME pFrameBase);

	void			GetFrameNamesRec(LPD3DXFRAME pFrame, std::vector<std::wstring> &FrameNames);

	void			SetInitialPoseRec(LPD3DXFRAME pFrame);

	virtual			ID3DXAllocateHierarchy* GetNewAlloc();


	std::wstring				m_strFilename;

	// pole na vypocet globalnich matic
	D3DXMATRIX					*m_pBoneMatrices;

	// zacatek hierarchie
	LPD3DXFRAME					m_pFrameRoot;

	// frame, ve kterem se nachazi mesh container
	LPD3DXFRAME					m_pFrameParent; 

	// frame, ktery je oznacen a bude uzivatelem rucne rotovan
	LPD3DXFRAME					m_pFrameSelected;
	D3DXMATRIX					m_matTMFrameSelected;

	// mesh container
	D3DXMESHCONTAINER_DERIVED*	m_pMeshContainer;

	bool						m_bOptimize;
	bool						m_bWeld;

	D3DXVECTOR3					m_vObjectCenter;
	float						m_fObjectRadius;

	int							m_NumBones;

	LPD3DXANIMATIONCONTROLLER	m_pAnimController;
	CSkinningMethod*			m_pSM;

	CMeshMenu*					m_pMeshMenu;					

	DWORD						m_dwBehaviorFlags;

	std::wstring				m_ErrMessage;

};






