#include "dxstdafx.h"

#include "CQuatMesh.h"
#include "CSkinningMethod.h"
#include "VertexDecl.h"




CQuatMesh::CQuatMesh(std::wstring filename, bool weld, bool optimize) : CMesh(filename, weld, optimize) 
{ 
	m_pBoneQuats = NULL; 
	m_pTranslations = NULL; 
	m_pHierarchy = NULL;
}




void CQuatMesh::AllocForBones(int NumBones)
{

//	CMesh::AllocForBones(NumBones);

	m_pBoneQuats = new D3DXQUATERNION[NumBones];
	m_pTranslations = new D3DXVECTOR4[NumBones];
	m_pHierarchy = new int[NumBones];
	memset(m_pHierarchy,0,NumBones*sizeof(int));

	m_NumBones = NumBones;

}

void CQuatMesh::FreeForBones()
{
//	CMesh::FreeForBones();

	delete[] m_pBoneQuats;
	delete[] m_pTranslations;
	delete[] m_pHierarchy;

	m_pBoneQuats = NULL;
	m_pTranslations = NULL;
}


void CQuatMesh::SetupBonePointersOnMeshAlloc(LPD3DXMESHCONTAINER pMeshContainerBase, int iBones)
{
//	CMesh::SetupBonePointersOnMeshAlloc(pMeshContainerBase,iBones);

	D3DXMESHCONTAINER_QUAT *pMeshContainer = (D3DXMESHCONTAINER_QUAT*)pMeshContainerBase;

	pMeshContainer->ppBoneQuatPtrs = new D3DXQUATERNION*[iBones];
	pMeshContainer->ppBoneTransPtrs = new D3DXVECTOR4*[iBones];

}

void CQuatMesh::SetupBonePointersOnMeshCB(LPD3DXMESHCONTAINER pMeshContainerBase, LPD3DXFRAME pFrameBase, int iBone)
{
//	CMesh::SetupBonePointersOnMeshCB(pMeshContainerBase,pFrameBase,iBone);

	D3DXMESHCONTAINER_QUAT *pMeshContainer = (D3DXMESHCONTAINER_QUAT*)pMeshContainerBase;
	D3DXFRAME_QUAT* pFrame = (D3DXFRAME_QUAT*) pFrameBase;

	pMeshContainer->ppBoneQuatPtrs[iBone] = &pFrame->CombinedQuat;
	pMeshContainer->ppBoneTransPtrs[iBone] = &pFrame->CombinedTrans;

}




// Prevod do matic do quaternionu
void CQuatMesh::UpdateFramesPreCB(LPD3DXFRAME pFrameBase,LPD3DXFRAME pFrameParent)
{

	D3DXFRAME_QUAT *pFrame = (D3DXFRAME_QUAT*)pFrameBase;
	D3DXFRAME_QUAT *pFrameParentDerived = (D3DXFRAME_QUAT*) pFrameParent;

	CMesh::UpdateFramesPreCB(pFrameBase,pFrameParent);

	D3DXQuaternionRotationMatrix(&pFrame->QuatRel,&pFrame->TransformationMatrix);
	pFrame->TransRel.x = pFrame->TransformationMatrix._41;
	pFrame->TransRel.y = pFrame->TransformationMatrix._42;
	pFrame->TransRel.z = pFrame->TransformationMatrix._43;

	D3DXQuaternionRotationMatrix(&pFrame->CombinedQuat,&pFrame->CombinedTransformationMatrix);
	pFrame->CombinedTrans.x = pFrame->CombinedTransformationMatrix._41;
	pFrame->CombinedTrans.y = pFrame->CombinedTransformationMatrix._42;
	pFrame->CombinedTrans.z = pFrame->CombinedTransformationMatrix._43;

	D3DXQuaternionMultiply(&pFrame->CombinedQuat,&pFrame->QuatOffset, &pFrame->CombinedQuat);

	if (pFrameParentDerived && D3DXQuaternionDot(&pFrame->CombinedQuat, &pFrameParentDerived->CombinedQuat) < 0)
		pFrame->CombinedQuat *= -1;


	
/*
	if (pFrameParentDerived && D3DXQuaternionDot(&pFrame->QuatRel, &pFrameParentDerived->QuatRel) < 0)
		pFrame->CombinedQuat *= -1;
*/
}




void CQuatMesh::SetBonesToEffect(IDirect3DDevice9 *pd3dDevice, UINT iAttrib, LPD3DXMESHCONTAINER pMeshContainerBase)
{
//	CMesh::SetBonesToEffect(iAttrib,pMeshContainerBase);
	D3DXMESHCONTAINER_QUAT *pMeshContainer = (D3DXMESHCONTAINER_QUAT*)pMeshContainerBase;
	HRESULT hr;
	UINT iMatrixIndex;
	UINT iPaletteEntry;
	LPD3DXBONECOMBINATION pBoneComb;

	pBoneComb = reinterpret_cast<LPD3DXBONECOMBINATION>(pMeshContainer->pBoneCombinationBuf->GetBufferPointer());

	// first calculate all the world matrices
	for (iPaletteEntry = 0; iPaletteEntry < pMeshContainer->NumPaletteEntries; ++iPaletteEntry)
	{
		iMatrixIndex = pBoneComb[iAttrib].BoneId[iPaletteEntry];
		if (iMatrixIndex != UINT_MAX)
		{

			m_pBoneQuats[iPaletteEntry] = *pMeshContainer->ppBoneQuatPtrs[iMatrixIndex];
//			D3DXQuaternionMultiply(&m_pBoneQuats[iPaletteEntry],&pMeshContainer->pBoneOffsetQuats[iMatrixIndex], pMeshContainer->ppBoneQuatPtrs[iMatrixIndex]);						

			m_pTranslations[iPaletteEntry] = *pMeshContainer->ppBoneTransPtrs[iMatrixIndex];
			
/*
			if (D3DXQuaternionDot(&m_pBoneQuats[0],&m_pBoneQuats[iPaletteEntry]) < 0)
				m_pBoneQuats[iPaletteEntry] *= -1;
*/		}
	}

	V( m_pSM->GetD3DXEffect()->SetVectorArray( "WorldQuatArray", (D3DXVECTOR4*) m_pBoneQuats, pMeshContainer->NumPaletteEntries) );
	V( m_pSM->GetD3DXEffect()->SetVectorArray( "WorldTrans", m_pTranslations, pMeshContainer->NumPaletteEntries) );
}






void CQuatMesh::RetrieveHierarchy(int &iNumber, LPD3DXFRAME pFrame)
{
	LPCSTR Name;

	if (pFrame == NULL) return;
	if (m_pMeshContainer->pSkinInfo == NULL) return;

	for ( int i=0; i<m_pMeshContainer->pSkinInfo->GetNumBones(); ++i)
	{
		Name = m_pMeshContainer->pSkinInfo->GetBoneName(i);
		if (pFrame->Name && !strcmp(pFrame->Name, Name) )
		{
			m_pHierarchy[i] = iNumber;
			++iNumber;
			break;
		}
	}

	if (pFrame->pFrameFirstChild != NULL)
	{
		RetrieveHierarchy(iNumber,pFrame->pFrameFirstChild);
	}

	if (pFrame->pFrameSibling != NULL)
	{
		RetrieveHierarchy(iNumber,pFrame->pFrameSibling);
	}


}







/*

void CQuatMesh::SortMatrixIndices(BYTE* pIndices,FLOAT* pWeights, DWORD NumInfl)
{
	int Index = NumInfl-1;
	BYTE IndexMatrix = pIndices[NumInfl-1];

	pWeights[NumInfl-1] = 1.0f;
	for(int i=0; i<NumInfl-1; ++i) pWeights[NumInfl-1] -= pWeights[i];

	for(int i=0; i<NumInfl; ++i)
	{
		if ( m_pHierarchy[pIndices[i]] >= m_pHierarchy[IndexMatrix] && pWeights[i] > 0.001)
		{
			IndexMatrix = pIndices[i];
			Index = i;
		}
	}

	// Vymenit s prvnim 
	if ( Index != 0 && pWeights[Index] > 0.001)
	{
		int BackupIndexMatrix = pIndices[0];
		float BackupWeight = pWeights[0];

		pIndices[0] = IndexMatrix;
		pWeights[0] = pWeights[Index];

		pIndices[Index] = BackupIndexMatrix;
		pWeights[Index] = BackupWeight;

	}
}
*/




void CQuatMesh::SortMatrixIndices(BYTE* pIndices,FLOAT* pWeights, DWORD NumInfl)
{
	FLOAT pWeightsPom[4];
	int Index = 0;
	BYTE IndexMatrix = pIndices[0];
		
	pWeightsPom[NumInfl-1] = 1.0f;
	for(int i=0; i<NumInfl-1; ++i) 
	{
		pWeightsPom[i] = pWeights[i];
		pWeightsPom[NumInfl-1] -= pWeights[i];
	}

	for(int i=0; i<NumInfl; ++i)
	{
		if ( m_pHierarchy[pIndices[i]] >= m_pHierarchy[IndexMatrix] && pWeightsPom[i] > 0.001)
		{
			IndexMatrix = pIndices[i];
			Index = i;
		}
	}

	// Vymenit s prvnim 
	if ( Index != 0 && pWeightsPom[Index] > 0.001)
	{
		int BackupIndexMatrix = pIndices[0];
		float BackupWeight = pWeightsPom[0];

		pIndices[0] = IndexMatrix;
		pWeightsPom[0] = pWeightsPom[Index];

		pIndices[Index] = BackupIndexMatrix;
		pWeightsPom[Index] = BackupWeight;

	}


	for(int i=0; i<3; ++i) 
	{
		pWeights[i] = pWeightsPom[i];
	}

}




DWORD CQuatMesh::GetBoneCombID(DWORD dwVertex, LPD3DXBONECOMBINATION pBoneComb)
{
	for(int i=0;;++i)
	{
		if ( pBoneComb[i].VertexStart <= dwVertex && 
			 dwVertex < pBoneComb[i].VertexStart + pBoneComb[i].VertexCount )
			return i;
	}

	return 0;

}



INT CQuatMesh::GetBoneIndexByName(LPCSTR BoneName)
{
	if (!BoneName || !strcmp(BoneName,"") || !m_pMeshContainer->pSkinInfo )
		return -1;

	INT cBones = m_pMeshContainer->pSkinInfo->GetNumBones();
	for (int iBone = 0; iBone < cBones; iBone++)
	{
		LPCSTR  pom = m_pMeshContainer->pSkinInfo->GetBoneName(iBone);
		if (!strcmp(pom,BoneName))
			return iBone;
	}

	return -1;

}




void CQuatMesh::RetrieveOffsetQuats(D3DXFRAME_QUAT *pFrame)
{
	if (!pFrame) return;

	
	INT iMatrixIndex = GetBoneIndexByName(pFrame->Name);
	if ( iMatrixIndex != -1)
		pFrame->QuatOffset = m_pMeshContainer->pBoneOffsetQuats[iMatrixIndex];
	else 
		pFrame->QuatOffset = D3DXQUATERNION(0.0f,0.0f,0.0f,1.0f);


	RetrieveOffsetQuats( (D3DXFRAME_QUAT*)pFrame->pFrameSibling);
	RetrieveOffsetQuats( (D3DXFRAME_QUAT*)pFrame->pFrameFirstChild);
}





void CQuatMesh::PrecomputeTranslations()
{
	Blend4TexUnLitVERTEX* pData;
	LPD3DXMESH pMesh = m_pMeshContainer->MeshData.pMesh;
	DWORD dwVertices = pMesh->GetNumVertices();
	BYTE* pIndices;
	FLOAT* pWeights;
	DWORD dwStride = pMesh->GetNumBytesPerVertex();
	DWORD dwBoneCombID;
	LPD3DXBONECOMBINATION pBoneComb;

	if (m_pMeshContainer->pBoneCombinationBuf == NULL) return;

	pBoneComb = reinterpret_cast<LPD3DXBONECOMBINATION>(m_pMeshContainer->pBoneCombinationBuf->GetBufferPointer());


	// locknout vertex buffer
	// a prenasobit vrchol matici
	// jakou vybrat: zatim tu s maximalnim indexem



	if (pMesh->LockVertexBuffer(0,(void**)&pData) == D3D_OK)
	{
		for(DWORD i=0; i<dwVertices; ++i)
		{
			pIndices = (BYTE*) &pData->indices;
			pWeights = (FLOAT*) &pData->blend1;


			dwBoneCombID = GetBoneCombID(i,pBoneComb);

			SortMatrixIndices(pIndices, pWeights, m_pMeshContainer->NumInfl);

			D3DXMATRIX matInv;
			D3DXVECTOR3 v,vOut;

			int iMatrixIndex = pBoneComb[dwBoneCombID].BoneId[pIndices[0]];
			v.x = m_pMeshContainer->pBoneOffsetMatrices[iMatrixIndex]._41;
			v.y = m_pMeshContainer->pBoneOffsetMatrices[iMatrixIndex]._42;
			v.z = m_pMeshContainer->pBoneOffsetMatrices[iMatrixIndex]._43;

			D3DXMatrixInverse(&matInv,NULL,&m_pMeshContainer->pBoneOffsetMatrices[iMatrixIndex]);
			D3DXVec3TransformNormal(&vOut,&v, &matInv);

			pData->position += vOut;

			pData = (Blend4TexUnLitVERTEX*)(((BYTE*)pData) + dwStride);
		}

		pMesh->UnlockVertexBuffer();
	}
}



HRESULT CQuatMesh::AfterLoad(IDirect3DDevice9* pd3dDevice)
{

	int iNumber = 0;
	RetrieveHierarchy(iNumber,m_pFrameRoot);

	RetrieveOffsetQuats((D3DXFRAME_QUAT*)m_pFrameRoot);

	PrecomputeTranslations();

	return S_OK;
	
}





void CQuatMesh::ProcessVerticesSkinned(VERTEX **ppVertices)
{
	BYTE* pIndices;
	FLOAT* pWeights;
	Blend4TexUnLitVERTEX* pData;
	LPD3DXMESH pMesh = m_pMeshContainer->MeshData.pMesh;
	D3DXMESHCONTAINER_QUAT* pMeshContainer = (D3DXMESHCONTAINER_QUAT*)m_pMeshContainer;
	DWORD dwVertices = pMesh->GetNumVertices();
	DWORD dwStride = pMesh->GetNumBytesPerVertex();
	DWORD dwBoneCombID;
	LPD3DXBONECOMBINATION pBoneComb;


	*ppVertices = new VERTEX[dwVertices];

	pBoneComb = reinterpret_cast<LPD3DXBONECOMBINATION>(m_pMeshContainer->pBoneCombinationBuf->GetBufferPointer());


	if (pMesh->LockVertexBuffer(0,(void**)&pData) == D3D_OK)
	{
		for(DWORD i=0; i<dwVertices; ++i)
		{
			pIndices = (BYTE*) &pData->indices;
			pWeights = (FLOAT*) &pData->blend1;


			dwBoneCombID = GetBoneCombID(i,pBoneComb);

			// vypocti quaternion blendenim
			D3DXQUATERNION QuatFinal(0.0f,0.0f,0.0f,0.0f);
			D3DXMATRIX MatFinal;
			INT iBoneIndex;

			for (int j=0; j<m_pMeshContainer->NumInfl; ++j)
			{
				iBoneIndex = pBoneComb[dwBoneCombID].BoneId[pIndices[j]];
				QuatFinal += *pMeshContainer->ppBoneQuatPtrs[iBoneIndex]*pWeights[j];
			}
/*
			D3DXQuaternionNormalize(&QuatFinal,&QuatFinal);
			D3DXMatrixRotationQuaternion(&MatFinal,&QuatFinal);

			D3DXVec3TransformCoord(&(*ppVertices)[i].position,&pData->position,&MatFinal);
			D3DXVec3TransformCoord(&(*ppVertices)[i].normal,&pData->normal,&MatFinal);

			// zjisti posunuti podle hlavni kosti
			D3DXVECTOR3 vTrans;
			iBoneIndex = pBoneComb[dwBoneCombID].BoneId[pIndices[0]];
			vTrans.x = pMeshContainer->ppBoneMatrixPtrs[iBoneIndex]->_41;
			vTrans.y = pMeshContainer->ppBoneMatrixPtrs[iBoneIndex]->_42;
			vTrans.z = pMeshContainer->ppBoneMatrixPtrs[iBoneIndex]->_43;

			(*ppVertices)[i].position += vTrans;
*/

			pData = (Blend4TexUnLitVERTEX*)(((BYTE*)pData) + dwStride);
		}

		pMesh->UnlockVertexBuffer();
	}
}