/*! \file 
    \brief Tda modelu pro metodu SJB
*/ 

#pragma once

#include "CMesh.h"


/// Struktura kosti pro tdu CQuatMesh
struct D3DXFRAME_QUAT : public D3DXFRAME_DERIVED
{
    /// Offsetov kvaterniony, odpovdaj ofsetovm maticm
	D3DXQUATERNION		QuatOffset;

	D3DXFRAME_QUAT()
	{
		QuatOffset = D3DXQUATERNION(0.0f,0.0f,0.0f,1.0f);
	}

};

/// Mesh kontejner pro tdu CQuatMesh
struct D3DXMESHCONTAINER_QUAT : public D3DXMESHCONTAINER_DERIVED
{
    /// Pole vech odkaz na kvaterniony v kostech
	D3DXQUATERNION** ppBoneQuatPtrs;
    
    /// Pole vech odkaz na translace v kostech    
	D3DXVECTOR4**	ppBoneTransPtrs;	

	D3DXMESHCONTAINER_QUAT()
	{
		ppBoneQuatPtrs = NULL;
		ppBoneTransPtrs = NULL;
	}


	~D3DXMESHCONTAINER_QUAT()
	{
		SAFE_DELETE_ARRAY( ppBoneQuatPtrs );
		SAFE_DELETE_ARRAY( ppBoneTransPtrs );
	}

};

/// Tda modelu pro metodu SJB
class CQuatMesh : public CMesh
{
public:

	CQuatMesh(std::wstring filename, bool weld, bool optimize=true);
	virtual ~CQuatMesh() {}

	virtual void	AllocForBones(int m_NumBoneMatricesMax);
	virtual void	FreeForBones();

	virtual D3DXFRAME_BASE*			GetNewFrame() { return new D3DXFRAME_QUAT; }
	virtual D3DXMESHCONTAINER_BASE*	GetNewMeshContainer() { return new D3DXMESHCONTAINER_QUAT; }

	// upravy po naloadovani modelu
	// v tomto pripade transformovat vertexy do prostoru prislusne kosti
	virtual HRESULT	AfterLoad(IDirect3DDevice9* pd3dDevice);

protected:

	virtual void	SetupBonePointersOnMeshAlloc(LPD3DXMESHCONTAINER pMeshContainerBase, int iBones);
	virtual void	SetupBonePointersOnMeshCB(LPD3DXMESHCONTAINER pMeshContainerBase, LPD3DXFRAME pFrameBase, int iBone);

	void			UpdateFramesPreCB(LPD3DXFRAME pFrameBase,LPD3DXFRAME pFrameParent);

	virtual void	SetBonesToEffect(IDirect3DDevice9 *pd3dDevice, UINT iAttrib, LPD3DXMESHCONTAINER pMeshContainerBase);

	virtual void	ProcessVerticesSkinned(VERTEX **ppVertices);



	void	SortMatrixIndices(BYTE* pIndices,FLOAT* pWeights, DWORD NumInfl);
	void	RetrieveHierarchy(int &iNumber, LPD3DXFRAME pFrame);
	void	RetrieveOffsetQuats(D3DXFRAME_QUAT* pFrame);
	void	PrecomputeTranslations();
	INT		GetBoneIndexByName(LPCSTR BoneName);
	DWORD	GetBoneCombID(DWORD dwVertex, LPD3DXBONECOMBINATION pBoneComb);

	int*						m_pHierarchy;
	D3DXQUATERNION*				m_pBoneQuats;
	D3DXVECTOR4*				m_pTranslations;


};






