#include "dxstdafx.h"


#include "CSWMesh.h"
#include "CAllocateHierarchySW.h"





ID3DXAllocateHierarchy* CSWMesh::GetNewAlloc()
{
	return new CAllocateHierarchySW(this);
}



HRESULT CSWMesh::Optimize(IDirect3DDevice9 *pd3dDevice)
{
	D3DXWELDEPSILONS Epsilons;
	HRESULT hr;
	LPD3DXBUFFER pVertexRemap;

	Epsilons.Position	= 0.001f;
	Epsilons.Normal		= 0.001f;

	if (m_bWeld)
	{
	
		V_RETURN(D3DXWeldVertices(m_pMeshContainer->pOrigMesh, D3DXWELDEPSILONS_WELDPARTIALMATCHES, &Epsilons, 
			m_pMeshContainer->pAdjacency, m_pMeshContainer->pAdjacency, NULL, &pVertexRemap));

		V_RETURN(m_pMeshContainer->pSkinInfo->Remap(m_pMeshContainer->pOrigMesh->GetNumVertices(), (DWORD*)pVertexRemap->GetBufferPointer()));
	}

	V_RETURN(m_pMeshContainer->pOrigMesh->OptimizeInplace(D3DXMESHOPT_VERTEXCACHE,m_pMeshContainer->pAdjacency,
		m_pMeshContainer->pAdjacency, NULL, &pVertexRemap) );

	V_RETURN(m_pMeshContainer->pSkinInfo->Remap(m_pMeshContainer->pOrigMesh->GetNumVertices(), (DWORD*)pVertexRemap->GetBufferPointer()));

	LPD3DXMESH pMesh;

	hr = m_pMeshContainer->pSkinInfo->ConvertToIndexedBlendedMesh
		(
		m_pMeshContainer->pOrigMesh,
		D3DXMESH_SYSTEMMEM, 
		m_pMeshContainer->NumPaletteEntries, 
		m_pMeshContainer->pAdjacency, 
		NULL, NULL, NULL,             
		&m_pMeshContainer->NumInfl,
		&m_pMeshContainer->NumAttributeGroups, 
		&m_pMeshContainer->pBoneCombinationBuf, 
		&pMesh);

	m_pMeshContainer->pOrigMesh->Release();
	m_pMeshContainer->pOrigMesh = pMesh;

	m_pMeshContainer->MeshData.pMesh->Release();

	m_pMeshContainer->pOrigMesh->CloneMeshFVF(
		D3DXMESH_SYSTEMMEM,
		m_pMeshContainer->pOrigMesh->GetFVF(),
		pd3dDevice,
		&m_pMeshContainer->MeshData.pMesh
	);


	return S_OK;
}



void CSWMesh::UpdateSkinnedMesh(D3DXMATRIX *pBoneMatrices, BYTE *pSource,BYTE * pDest, UINT iAttrib, LPD3DXBONECOMBINATION pBoneComb, DWORD dwMaxInfl)
{
	D3DXVECTOR3 *pPosition, *pPosOut;
	D3DXVECTOR3 *pNormal, *pNormOut;
	BYTE *pMatIndices;
	D3DXVECTOR3 pomVec, pomNorm;
	FLOAT fLastBlend;
	FLOAT *pBlend;
	DWORD dwVertSize = sizeof(Blend4TexUnLitVERTEX) - (4-dwMaxInfl)*sizeof(FLOAT);
	DWORD dwBlendOfs = sizeof(D3DXVECTOR3);
	DWORD dwIndOfs  = sizeof(D3DXVECTOR3) + 3*sizeof(FLOAT) - (4-dwMaxInfl)*sizeof(FLOAT);
	DWORD dwNormOfs = sizeof(D3DXVECTOR3) + 3*sizeof(FLOAT) - (4-dwMaxInfl)*sizeof(FLOAT) + sizeof(DWORD);

	for (UINT i=pBoneComb[iAttrib].VertexStart; i<pBoneComb[iAttrib].VertexStart+pBoneComb[iAttrib].VertexCount; ++i)
	{
		DWORD dwOfs = dwVertSize*i;

		pMatIndices = (BYTE*)&pSource[dwOfs + dwIndOfs];
		pBlend = (FLOAT*)&pSource[dwOfs + dwBlendOfs];
		pPosition = (D3DXVECTOR3*)&pSource[dwOfs];
		pNormal = (D3DXVECTOR3*)&pSource[dwOfs + dwNormOfs];

		pPosOut = (D3DXVECTOR3*)&pDest[dwOfs];
		pNormOut = (D3DXVECTOR3*)&pDest[dwOfs + dwNormOfs];

		pPosOut->x = pPosOut->y = pPosOut->z = 0.0f;
		pNormOut->x = pNormOut->y = pNormOut->z = 0.0f;

		fLastBlend = 1.0f;

		for (UINT j=0; j<dwMaxInfl-1; j++)
		{
			D3DXVec3TransformCoord(&pomVec, pPosition, &pBoneMatrices[pMatIndices[j]]);
			D3DXVec3TransformNormal(&pomNorm, pNormal, &pBoneMatrices[pMatIndices[j]]);
			*pPosOut			+= pomVec*pBlend[j];
			*pNormOut			+= pomNorm*pBlend[j];
			fLastBlend			-= pBlend[j];
		}
		D3DXVec3TransformCoord(&pomVec, pPosition, &pBoneMatrices[pMatIndices[dwMaxInfl-1]]);
		D3DXVec3TransformNormal(&pomNorm, pNormal, &pBoneMatrices[pMatIndices[dwMaxInfl-1]]);
		*pPosOut	+= pomVec*fLastBlend;
		*pNormOut	+= pomNorm*fLastBlend;
	}
}




void CSWMesh::SetBonesToEffect( IDirect3DDevice9 *pd3dDevice, UINT iAttrib, LPD3DXMESHCONTAINER pMeshContainerBase)
{
	D3DXMESHCONTAINER_DERIVED *pMeshContainer = (D3DXMESHCONTAINER_DERIVED*)pMeshContainerBase;
	UINT iMatrixIndex;
	UINT iPaletteEntry;
	LPD3DXBONECOMBINATION pBoneComb;

	pBoneComb = reinterpret_cast<LPD3DXBONECOMBINATION>(pMeshContainer->pBoneCombinationBuf->GetBufferPointer());

	// first calculate all the world matrices
	for (iPaletteEntry = 0; iPaletteEntry < pMeshContainer->NumPaletteEntries; ++iPaletteEntry)
	{
		iMatrixIndex = pBoneComb[iAttrib].BoneId[iPaletteEntry];
		if (iMatrixIndex != UINT_MAX)
		{
			D3DXMatrixMultiply(&m_pBoneMatrices[iPaletteEntry],&pMeshContainer->pBoneOffsetMatrices[iMatrixIndex], pMeshContainer->ppBoneMatrixPtrs[iMatrixIndex]);						
		}
	}
}




void CSWMesh::DrawMeshContainer( IDirect3DDevice9 *pd3dDevice, LPD3DXMESHCONTAINER pMeshContainerBase, LPD3DXFRAME pFrameBase)
{
	D3DXMESHCONTAINER_DERIVED *pMeshContainer = (D3DXMESHCONTAINER_DERIVED*)pMeshContainerBase;
	D3DXFRAME_DERIVED *pFrame = (D3DXFRAME_DERIVED*)pFrameBase;
	HRESULT hr;
	UINT iAttrib;
	LPD3DXBONECOMBINATION pBoneComb;

	if (pMeshContainer->pSkinInfo == NULL) 
	{

		V( pd3dDevice->SetTransform(D3DTS_WORLD, &pFrame->CombinedTransformationMatrix) );

		for (int iMaterial = 0; iMaterial < pMeshContainer->NumMaterials; iMaterial++)
		{
			V( pd3dDevice->SetMaterial( &pMeshContainer->pMaterials[iMaterial].MatD3D ) );
			V( pd3dDevice->SetTexture( 0, pMeshContainer->ppTextures[iMaterial] ) );
			V( pMeshContainer->MeshData.pMesh->DrawSubset(iMaterial) );
		}
		return;
	}

	BYTE* pSourceVert;
	BYTE* pDestVert;

	D3DXMATRIX matId;

	D3DXMatrixIdentity(&matId);
	V( pd3dDevice->SetTransform(D3DTS_WORLD, &matId) );

	pMeshContainer->pOrigMesh->LockVertexBuffer(D3DLOCK_READONLY,(void**)&pSourceVert);
	pMeshContainer->MeshData.pMesh->LockVertexBuffer(0,(void**)&pDestVert);

	pBoneComb = reinterpret_cast<LPD3DXBONECOMBINATION>(pMeshContainer->pBoneCombinationBuf->GetBufferPointer());
	for (iAttrib = 0; iAttrib < pMeshContainer->NumAttributeGroups; iAttrib++)
	{ 
		SetBonesToEffect(pd3dDevice, iAttrib,pMeshContainer);
		UpdateSkinnedMesh(m_pBoneMatrices,pSourceVert,pDestVert,iAttrib,pBoneComb,GetNumBonesPerVertex());
	}

	pMeshContainer->pOrigMesh->UnlockVertexBuffer();
	pMeshContainer->MeshData.pMesh->UnlockVertexBuffer();


	for (iAttrib = 0; iAttrib < pMeshContainer->NumAttributeGroups; iAttrib++)
	{ 

		// Sum of all ambient and emissive contribution
		D3DXCOLOR color1(pMeshContainer->pMaterials[pBoneComb[iAttrib].AttribId].MatD3D.Ambient);
		D3DXCOLOR color2(.25, .25, .25, 1.0);
		D3DXCOLOR ambEmm;
		D3DXColorModulate(&ambEmm, &color1, &color2);
		ambEmm += D3DXCOLOR(pMeshContainer->pMaterials[pBoneComb[iAttrib].AttribId].MatD3D.Emissive);

		V( pd3dDevice->SetMaterial( &pMeshContainer->pMaterials[pBoneComb[iAttrib].AttribId].MatD3D ) );
		V( pd3dDevice->SetTexture( 0, pMeshContainer->ppTextures[pBoneComb[iAttrib].AttribId] ) );
		V( pMeshContainer->MeshData.pMesh->DrawSubset(iAttrib) );
	}

}




