/*! \file 
    \brief Zkladn interface modelu.
*/ 

#pragma once

#include "VertexDecl.h"
#include "CAllocateHierarchy.h"
#include <vector>

class CSkinningMethod;
class CMeshMenu;

/// Zkladn interface modelu.
/// Pes tento interface komunikuje program s modelem.
class CVirtualMesh
{
public:

	virtual ~CVirtualMesh() {}

	/// Reakce na vytvoen D3DDevice
	virtual HRESULT OnCreateDevice(IDirect3DDevice9* pd3dDevice, DWORD dwBehaviorFlags) =0;
        
    /// Reakce na ruen D3DDevice
	virtual void	OnDestroyDevice() =0;

    /// Vrt odkaz na koenovou kost v hierarchii skeletonu
	virtual LPD3DXFRAME					GetRootFrame() =0;
    
    /// Vrt mesh kontejner, kter obsahuje data o modelu
	virtual D3DXMESHCONTAINER_BASE*		GetMeshContainer() =0;
    
    /// Vrt odkaz na kost podle jmna
	virtual LPD3DXFRAME					GetFrameByName(LPCSTR BoneName) =0;

    /// Vrt seznam vech kost
	virtual	void	GetFrameNames(std::vector<std::wstring> &FrameNames) =0;
        
    /// Vykresl model
	virtual void	Render(IDirect3DDevice9* pd3dDevice) =0;

    /// Nastav as v animan sekvenci
	virtual void  	SetTime(double fElapsedTime) =0;
    
    /// Aktualizuje as v animan sekvenci
	virtual void	AdvanceTime(double fElapsedTime) =0;
    
    /// Vrt souasn as animace
	virtual double	GetAnimationTime() =0;

    /// Nastaven metody skinningu
	virtual void	SetSkinningMethod(CSkinningMethod* pSM) =0;

    /// Vrt polomr obalov koule
	virtual float	GetObjectRadius() =0;
    
    /// Vrt sted obalov koule
	virtual D3DXVECTOR3 GetObjectCenter() =0;
    
    /// Vrt transformovan vrcholy podle nastaven konfigurace skeletonu
	virtual void	ProcessVertices(VERTEX **ppVertices) =0;
    
    /// Nastav vybranou kost podle jmna
	virtual void	SelectFrame(LPCSTR BoneName) =0;
    
    /// Transformace vybran kosti matic
	virtual void	TransformSelectedFrame(D3DXMATRIX *mat) =0;

    /// Nastav referenn pzu na skeleton
	virtual void	SetInitialPose() =0;

    /// Vrt poet vrchol
	virtual INT		GetNumVertices() =0;
    
    /// Vrt poet trojhelnk
	virtual INT		GetNumFaces()	 =0;
    
    /// Vrt maximln poet kost, kter ovlivuj jeden vrchol
	virtual INT		GetNumBonesPerVertex() =0;

    /// Vrt speciln nabdku pro model
	virtual CMeshMenu* GetMeshMenu() =0;

    /// Vrt chybovou hlku o prbhu renderingu
	virtual std::wstring GetErrMessage() =0;


};




/// Interface pro zptn voln pi nahrvn modelu ze souboru.
class CAllocationMeshCB
{
public:

	virtual ~CAllocationMeshCB() {}
        
	/// Alokace dat potebnch pro kosti
	virtual void	AllocForBones(int m_NumBoneMatricesMax) {}
        
    /// Uvolnn dat potebnch pro kosti
	virtual void	FreeForBones() {}
        
    /// Vytvoen struktury kosti
	virtual D3DXFRAME_BASE*			GetNewFrame() =0;
        
    /// Vytvoen mesh kontejneru, kter uchovv data modelu
	virtual D3DXMESHCONTAINER_BASE*	GetNewMeshContainer() =0;

};

