/*! \file 
    \brief Funkce main()
*/ 

//--------------------------------------------------------------------------------------
// File: SkinnedMesh.cpp
//
// Starting point for new Direct3D applications
//
// Copyright (c) Microsoft Corporation. All rights reserved.
//--------------------------------------------------------------------------------------
#include "dxstdafx.h"
#include "resource.h"
#include "CApplication.h"
#include "CMesh.h"
#include "CSWMesh.h"
#include "CFixedMesh.h"
#include "CFixedIndexedMesh.h"
#include "CQuatMesh.h"
#include "CExpMesh.h"
#include "CSkinningMethod.h"
#include <iostream>
#include <vector>
#include <string>


std::wstring MESHFILENAME = L"cylindr2.x";
bool   WELD = true;



//--------------------------------------------------------------------------------------
// Entry point to the program. Initializes everything and goes into a message processing 
// loop. Idle time is used to render the scene.
//--------------------------------------------------------------------------------------
INT WINAPI wWinMain( HINSTANCE, HINSTANCE, LPWSTR cmdline, int )
{ 

	std::vector<std::wstring> commands;
	D3DCOLOR bgColor = D3DCOLOR_ARGB(0, 125, 125, 125);

	CApplication::ParseCommandLine(commands, cmdline);


	CSkinningMethod* methods[] =
	{
		new CSkinningMethod(L"",0,L"Vertex Blending - SW",L"",1),
		new CSkinningMethod(L"",0,L"Vertex Blending - FFP1",L"",2),
		new CSkinningMethod(L"",0,L"Vertex Blending - FFP2",L"",3),
		new CSkinningMethod(L"Shaders/skin_vertex_blending.fx",D3DXSHADER_PREFER_FLOW_CONTROL,L"Vertex Blending - VSP1",L"",0),
		new CSkinningMethod(L"Shaders/skin_vertex_blending.fx",D3DXSHADER_AVOID_FLOW_CONTROL,L"Vertex Blending - VSP2",L"",0),
		new CSkinningMethod(L"Shaders/skin_vertex_blending_matrix.fx",D3DXSHADER_AVOID_FLOW_CONTROL,L"Vertex Blending - VSP3",L"",0),
		new CSkinningMethod(L"Shaders/skin_spherical_joint_blending.fx",D3DXSHADER_AVOID_FLOW_CONTROL,L"Spherical Joint Blending",L"",4),
		//new CSkinningMethod(L"Shaders/skin_spherical_joint_blending_debug.fx",D3DXSHADER_AVOID_FLOW_CONTROL,L"Spherical Joint Blending + Debug",L"",5),
		new CSkinningMethod(L"Shaders/skin_sjb_examples.fx",D3DXSHADER_AVOID_FLOW_CONTROL,L"Spherical Joint Blending By Example",L"",5),
		NULL
	};

	if (commands[0] == L"" ) 
		return S_OK; 
	else MESHFILENAME = commands[0].c_str();

	if (commands[1] == L"noweld" ) WELD = false;

	if (commands[2] != L"")
	{
		bgColor = _wtoi(commands[2].c_str());
	}



	CVirtualMesh* meshes[] =
	{
		new CMesh(MESHFILENAME, WELD),
		new CSWMesh(MESHFILENAME, WELD),
		new CFixedMesh(MESHFILENAME, WELD),
		new CFixedIndexedMesh(MESHFILENAME, WELD),
		new CQuatMesh(MESHFILENAME, WELD),
//		new CQuatMeshDebug(MESHFILENAME, WELD),
		new CExpMesh(MESHFILENAME, WELD),
		NULL
	};


	CApplication::RegisterMethods(methods,meshes);
	CApplication::SetBackgroundColor(bgColor);



	return CApplication::Run();
}
